/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.exchange.admin;

import java.io.Serializable;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import org.apache.commons.lang.StringUtils;

import com.funambol.admin.AdminException;
import com.funambol.admin.ui.ConnectorManagementPanel;

/**
 * This class implements the connector configuration panel
 * for a Exchange Connector component
 *
 *
 * @version $Id: ExchangeConnectorConfigPanel.java,v 1.1 2006/08/08 07:52:29 gibi Exp $
 */
public class ExchangeConnectorConfigPanel
               extends ConnectorManagementPanel
               implements Serializable {

    // --------------------------------------------------------------- Constants

    public static final String PORT_ALLOWED_CHARS =
            "0123456789"                                                       ;

    public static final String NAME_ALLOWED_CHARS =
            "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.";

    // ------------------------------------------------------------ Private data


    private JTextField seccServerValue;
    private JTextField seccPortValue;
    private JTextField exchangeServerValue;
    private JCheckBox useSSLValue;
    private JLabel keyStoreFileNameLabel;
    private JTextField keyStoreFileNameValue;
    private JLabel keyStorePasswordLabel;
    private JTextField keyStorePasswordValue;

    private JButton confirmButton = new JButton();

    // ------------------------------------------------------------ Constructors
    public ExchangeConnectorConfigPanel() {
      init();
    }

    // --------------------------------------------------------- Private methods

    /**
     * Create the panel
     */
    private void init() {

        JLabel title, seccServerLabel, seccPortLabel, exchangeServerLabel;
        JPanel seccPanel, exchangePanel, sslPanel;


        title = new JLabel();
        exchangePanel = new JPanel();
        seccPanel = new JPanel();
        sslPanel = new JPanel();

        seccServerLabel = new JLabel();
        seccServerValue = new JTextField();
        seccPortLabel = new JLabel();
        seccPortValue = new JTextField();
        useSSLValue = new JCheckBox();
        keyStoreFileNameLabel = new JLabel();
        keyStoreFileNameValue = new JTextField();
        keyStorePasswordLabel = new JLabel();
        keyStorePasswordValue = new JTextField();



        exchangeServerLabel   = new JLabel();
        exchangeServerValue   = new JTextField();

        setLayout(null);

        title.setFont(titlePanelFont);
        title.setText("Funambol Exchange Connector");
        title.setBounds(new Rectangle(14, 5, 316, 28));
        title.setAlignmentX(SwingConstants.CENTER);
        title.setBorder(new TitledBorder(""));

        seccPanel.setLayout(null);
        seccPanel.setBorder(new TitledBorder("HTTP Server Configuration"));

        seccServerLabel.setText("Server:");
        seccPanel.add(seccServerLabel);
        seccServerLabel.setBounds(10, 20, 116, 15);
        seccPanel.add(seccServerValue);
        seccServerValue.setBounds(150, 20, 220, 19);
        seccServerValue.setFont(defaultFont);

        seccPortLabel.setText("Port:");
        seccPanel.add(seccPortLabel);
        seccPortLabel.setBounds(10, 50, 131, 15);
        seccPanel.add(seccPortValue);
        seccPortValue.setBounds(150, 50, 220, 19);
        seccPortValue.setFont(defaultFont);

        add(seccPanel);
        seccPanel.setBounds(10, 50, 380, 80);

        exchangePanel.setLayout(null);
        exchangePanel.setBorder(new TitledBorder("WebDav Message Configuration"));

        exchangeServerLabel.setText("Server: ");
        exchangePanel.add(exchangeServerLabel);
        exchangeServerLabel.setBounds(10, 20, 150, 19);
        exchangePanel.add(exchangeServerValue);
        exchangeServerValue.setBounds(150, 20, 220, 19);
        exchangeServerValue.setFont(defaultFont);

        add(exchangePanel);
        exchangePanel.setBounds(10, 150, 380, 55);

        //the ssl option panel

        sslPanel.setLayout(null);
        sslPanel.setBorder(new TitledBorder("SSL"));

        useSSLValue.setText("Use SSL");
        useSSLValue.setSelected(false);
        sslPanel.add(useSSLValue);
        useSSLValue.setBounds(5, 20, 150, 19);

        useSSLValue.addActionListener(
                new ActionListener()
                {
                    public void actionPerformed(ActionEvent acEvent)
                    {
                        boolean enabled = useSSLValue.isSelected();
                        keyStoreFileNameLabel.setEnabled(enabled);
                        keyStoreFileNameValue.setEnabled(enabled);
                        keyStorePasswordLabel.setEnabled(enabled);
                        keyStorePasswordValue.setEnabled(enabled);
                    }
                }
        );

        keyStoreFileNameLabel.setText("Key Store File: ");
        sslPanel.add(keyStoreFileNameLabel);
        keyStoreFileNameLabel.setEnabled(false);
        keyStoreFileNameLabel.setBounds(10, 50, 131, 15);
        sslPanel.add(keyStoreFileNameValue);
        keyStoreFileNameValue.setEnabled(false);
        keyStoreFileNameValue.setBounds(150, 50, 220, 19);
        keyStoreFileNameValue.setFont(defaultFont);

        keyStorePasswordLabel.setText("Key Store Password: ");
        sslPanel.add(keyStorePasswordLabel);
        keyStorePasswordLabel.setEnabled(false);
        keyStorePasswordLabel.setBounds(10, 80, 131, 15);
        sslPanel.add(keyStorePasswordValue);
        keyStorePasswordValue.setEnabled(false);
        keyStorePasswordValue.setBounds(150, 80, 220, 19);
        keyStorePasswordValue.setFont(defaultFont);


        add(sslPanel);
        sslPanel.setBounds(10, 220, 380, 110);

        confirmButton.setFont(defaultFont);
        confirmButton.setText("Save");
        add(confirmButton);
        confirmButton.setBounds(160, 350, 70, 25);

        confirmButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event ) {
                try {
                    validateValues();
                    getValues();
                    ExchangeConnectorConfigPanel.this.actionPerformed(
                             new ActionEvent(ExchangeConnectorConfigPanel.this,
                                             ACTION_EVENT_UPDATE,
                                             event.getActionCommand()));
                } catch (Exception e) {
                    notifyError(new AdminException(e.getMessage()));
                }
            }
        });



        //
        // Setting font...
        //
        Component[] components = getComponents();
        for (int i=0; (components != null) && (i<components.length); ++i) {
            components[i].setFont(defaultFont);
        }

        //
        // We add it as the last one so that the font won't be changed
        //
        add(title);
  }

  /**
   *
   */
  public void updateForm() {

      ExchangeConnectorConfig exchangeConf = (ExchangeConnectorConfig)getConfiguration();

      seccServerValue.setText(exchangeConf.getSeccServer());
      seccPortValue.setText(String.valueOf(exchangeConf.getSeccPort()));
      exchangeServerValue.setText(exchangeConf.getExchangeServer());
      useSSLValue.setSelected(exchangeConf.getUseSSL());
      keyStoreFileNameLabel.setEnabled(exchangeConf.getUseSSL());
      keyStoreFileNameValue.setEnabled(exchangeConf.getUseSSL());
      keyStoreFileNameValue.setText(exchangeConf.getKeyStoreFileName());
      keyStorePasswordLabel.setEnabled(exchangeConf.getUseSSL());
      keyStorePasswordValue.setEnabled(exchangeConf.getUseSSL());
      keyStorePasswordValue.setText(exchangeConf.getKeyStorePassword());

  }

  // ----------------------------------------------------------- Private methods

  /**
   * Checks if the values provided by the user are all valid. In caso of errors,
   * a IllegalArgumentException is thrown.
   *
   * @throws IllegalArgumentException if:
   *         <ul>
   *         <li>anyvalue is not in the form <host>:<port>
   *         </ul>
   */
  private void validateValues() throws IllegalArgumentException {
      String value;

      // SECC server
      value = seccServerValue.getText().trim();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'SECC Server' cannot be empty");
      }

      // SECC port
      value = seccPortValue.getText().trim();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'SECC Port' cannot be empty");
      }

      if (!StringUtils.containsOnly(value, PORT_ALLOWED_CHARS.toCharArray())) {
          throw new
            IllegalArgumentException(
                "Only the following characters are " +
                "allowed for field 'SECC Port': \n"       +
                PORT_ALLOWED_CHARS                   );
      }

      // Exchange server
      value = exchangeServerValue.getText().trim();
      if (StringUtils.isEmpty(value)) {
        throw new IllegalArgumentException("Field 'Exchange server' cannot be empty");
      }

      //SSL
      value = keyStoreFileNameValue.getText().trim();
      if (useSSLValue.isSelected() && StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'Keystore File' cannot be empty");
      }


  }

  /**
   * Set properties with the values provided by the user.
   */
  private void getValues() {

      ExchangeConnectorConfig exchangeConf = (ExchangeConnectorConfig)getConfiguration();

      exchangeConf.setSeccServer(seccServerValue.getText().trim());
      exchangeConf.setSeccPort(Integer.parseInt(seccPortValue.getText().trim()));
      exchangeConf.setExchangeServer(exchangeServerValue.getText().trim());
      exchangeConf.setUseSSL(useSSLValue.isSelected());
      exchangeConf.setKeyStoreFileName(useSSLValue.isSelected()?keyStoreFileNameValue.getText():null);
      exchangeConf.setKeyStorePassword(useSSLValue.isSelected()?keyStorePasswordValue.getText():null);



  }

}