/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.exchange.engine.source;

import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.engine.source.SyncContext;
import com.funambol.framework.engine.source.SyncSourceException;
import com.funambol.framework.server.Sync4jDevice;

import java.util.TimeZone;
import java.util.logging.Level;

/**
 * This class implements common methods of<br>
 * ExchangeCalendarSyncSource,
 * ExchangeContactSyncSource,
 * ExchangeNoteSyncSource, and
 * ExchangeTaskSyncSource.
 *
 * @version $Id: ExchangeSyncSource.java,v 1.1 2006/08/08 07:52:15 gibi Exp $
 */
public abstract class ExchangeSyncSource
        extends com.funambol.exchange.engine.cache.ExchangeCachedSyncSource
        implements java.io.Serializable {

    protected static final String ITEM_ADD = "A";
    protected static final String ITEM_REMOVE = "D";
    protected static final String ITEM_UPDATE = "U";

    protected static final String ITEM_ADD_ERROR = "AE";
    protected static final String ITEM_REMOVE_ERROR = "DE";
    protected static final String ITEM_UPDATE_ERROR = "UE";

    protected static final String TYPE_VCARD = "text/x-vcard";
    protected static final String TYPE_ICAL = "text/x-vcalendar";
    protected static final String TYPE_SIFC = "text/x-s4j-sifc";
    protected static final String TYPE_SIFE = "text/x-s4j-sife";
    protected static final String TYPE_SIFN = "text/x-s4j-sifn";
    protected static final String TYPE_SIFT = "text/x-s4j-sift";

    protected String deviceTimeZoneDescr = null;
    protected TimeZone deviceTimeZone = null;

    protected String deviceCharset = null;

    protected int conflictResolution = 0;

    /**
     * Property schedule
     */
    private String schedule;

    /**
     * Getter for property schedule.
     *
     * @return Value of property schedule.
     */
    public String getSchedule() {
        return schedule;
    }

    /**
     * Setter for property schedule.
     *
     * @param schedule New value of property schedule.
     */
    public void setSchedule(String schedule) {
        this.schedule = schedule;
    }

    /**
     * Property scheduleEnabled
     */
    private boolean scheduleEnabled;

    /**
     * Getter for property scheduleEnabled.
     *
     * @return Value of property scheduleEnabled.
     */
    public boolean isSchedulEnablede() {
        return scheduleEnabled;
    }

    /**
     * Setter for property schedule.
     *
     * @param scheduleEnabled New value of property schedule.
     */
    public void setScheduleEnabled(boolean scheduleEnabled) {
        this.scheduleEnabled = scheduleEnabled;
    }

    public void beginSync(SyncContext context) throws SyncSourceException {

        Sync4jDevice device;
        String timezone;

        device = context.getPrincipal().getDevice();
        timezone = device.getTimeZone();
        if (device.getConvertDate()) {
            if (timezone != null && timezone.length() > 0) {
                deviceTimeZoneDescr = timezone;
                deviceTimeZone = TimeZone.getTimeZone(deviceTimeZoneDescr);
            }
        }

        this.conflictResolution = context.getConflictResolution();

        super.beginSync(context);

        if (log.isLoggable(Level.INFO)){
            log.info("Device Timezone: "+ deviceTimeZoneDescr);
        }
    }

    /**
     *
     */
    public void setOperationStatus(String operation,
                                   int statusCode,
                                   SyncItemKey[] keys) {

        if (log.isLoggable(Level.FINEST)) {

            StringBuffer message = new StringBuffer("Received status code '");
            message.append(statusCode).append("' for a '").
                    append(operation).append("'").append(" for this items: ");

            for (int i = 0; i < keys.length; i++) {
                message.append("\n- ").append(keys[i].getKeyAsString());
            }

            log.finest(message.toString());
        }
    }


}
