/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.exchange.items.calendar.manager;

import java.io.IOException;

import java.util.TimeZone;

import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.server.store.NotFoundException;

import com.funambol.exchange.DataAccessException;
import com.funambol.exchange.items.calendar.dao.CalendarDAO;
import com.funambol.exchange.items.calendar.model.Calendar;
import com.funambol.exchange.items.common.dao.ItemDAO;
import com.funambol.exchange.items.common.model.Item;
import com.funambol.exchange.util.DateTools;
import com.funambol.exchange.util.StringTools;
import com.funambol.exchange.xml.XmlParser;

/*
 * This class defines methods to access calendars data
 * in exchange server datastore
 *
 * @version $Id: CalendarManager.java,v 1.1 2006/08/08 07:47:27 gibi Exp $
 *
 **/
public class CalendarManager {

    //------------------------------------------------------------- Private Data

    private CalendarDAO cd   = null ;

    private ItemDAO     id   = null ;

    // The timezone to use (if required) in the dates conversion
    private TimeZone timezone = null;

    //--------------------------------------------------------------- Costructor

    public CalendarManager(String host           ,
                           int    port           ,
                           String exchangeServer ,
                           TimeZone timezone     )
        throws DataAccessException {

        this.cd = new CalendarDAO (host           ,
                                   port           ,
                                   exchangeServer ,
                                   timezone       );

        this.id = new ItemDAO (host,
                               port,
                               exchangeServer);

        this.timezone = timezone;
    }

    //----------------------------------------------------------- Public methods

    /**
     * get all items about principalId, sourceURI
     *
     * @param sourceURI
     * @param principalId
     *
     * @return items about principalId, sourceURI
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Item[] getAllItems(String   sourceURI,
                              long     principalId)
    throws DataAccessException {

        Item[] items = id.getLocalItems(sourceURI,
                                        principalId);

        return items;
    }

    /**
     * get calendar twin
     *
     * @param calendar the calendar object
     * @param principalId
     * @param sourceURI
     *
     * @return find contact
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Item[] getTwinItems(Calendar calendar    ,
                               long     principalId ,
                               String   sourceURI)
    throws DataAccessException {


        long crc = StringTools.getCRCFromString(calendar.getDtStart().getPropertyValueAsString()  +
                                                calendar.getDtEnd().getPropertyValueAsString()    +
                                                calendar.getLocation().getPropertyValueAsString() );

        Item[] items = id.getTwins(crc           ,
                                   principalId   ,
                                   sourceURI);
        return items;
    }

    /**
     * get calendar by key
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find calendar
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Calendar getCalendarById(String username    ,
                                    String credentials ,
                                    String id          ,
                                    String exchangeFolder   )
        throws DataAccessException, NotFoundException {

        Calendar[] calendars = null;

        calendars = this.cd.getCalendars(username          ,
                                         credentials       ,
                                         new String[] {id} ,
                                         exchangeFolder);

        if (calendars == null || !(calendars.length > 0)) {
            throw new NotFoundException("Calendar with id " + id + " not found");
        }

        return calendars[0];

    }

    /**
     * get Calendar array by username, credentials, ids, exchangeFolder
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return find calendars
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Calendar[] getCalendars(String username    ,
                                   String credentials ,
                                   String ids[]       ,
                                   String exchangeFolder   )
        throws DataAccessException {

        Calendar[] calendars = null;

        calendars = this.cd.getCalendars(username        ,
                                         credentials     ,
                                         ids             ,
                                         exchangeFolder);

        return calendars;

    }

    /**
     * get Calendar array with setting fields only for twins
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find items
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Calendar[] getCalendarsForTwin(String   username       ,
                                          String   credentials    ,
                                          String[] ids            ,
                                          String   exchangeFolder
                                          )
    throws DataAccessException {

        return cd.getCalendarsForTwin(username    ,
                                      credentials ,
                                      ids         ,
                                      exchangeFolder);
    }

    /**
     * add calendar
     *
     * @param calendar
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new calendar
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Calendar setCalendar(Calendar calendar    ,
                                String   username    ,
                                String   credentials ,
                                String   exchangeFolder
                              )
        throws DataAccessException {

        return this.cd.setCalendar(calendar    ,
                                   username    ,
                                   credentials ,
                                   exchangeFolder   );

    }

    /**
     * remove calendar
     *
     * @param calendar
     * @param username
     * @param credential
     * @param exchangeFolder
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public void removeCalendar(Calendar calendar  ,
                               String   username  ,
                               String   credential,
                               String   exchangeFolder)
        throws DataAccessException {

        this.cd.removeCalendar(calendar, username, credential, exchangeFolder);

    }

}
