/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.exchange.items.common.manager;

import com.funambol.exchange.DataAccessException;
import com.funambol.exchange.items.common.dao.ItemDAO;
import com.funambol.exchange.items.common.model.Item;

/*
 * This class defines methods to access commons items data
 * in database source and exchange datastore
 *
 * @version $Id: ItemManager.java,v 1.1 2006/08/08 07:48:18 gibi Exp $
 *
 **/
public class ItemManager {

    //------------------------------------------------------------- Private Data

    ItemDAO id = null ;

    //--------------------------------------------------------------- Costructor

    public ItemManager ()
        throws DataAccessException {

        this.id = new ItemDAO ();

    }

    public ItemManager (String  host           ,
                        int     port           ,
                        String  exchangeServer)
    throws DataAccessException {

        this.id = new ItemDAO (host           ,
                               port           ,
                               exchangeServer);
    }

    //----------------------------------------------------------- Public methods

    /**
     * get all local items
     *
     * @param sourceURI
     * @param principal
     *
     * @return array of find items
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Item[] getLocalItems (String sourceURI, long principal)
    throws DataAccessException {

         return id.getLocalItems (sourceURI, principal);

    }

     /**
      * get href
      *
      * @param itemId
      * @param principal
      * @param sourceURI
      *
      * @return href
      *
      * @throws com.funambol.exchange.DataAccessException
      **/
     public String getHref (String itemId, long principal, String sourceURI)
     throws DataAccessException {

          return id.getHref (itemId, principal, sourceURI);

     }

    /**
     * add local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     * @throws com.funambol.exchange.DataAccessException
     **/
    public void addItems (Item[] localItems ,
                          String sourceURI  ,
                          long   principal  )
    throws DataAccessException {

         id.addItems (localItems, sourceURI, principal);

    }

    /**
     * update local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     * @throws com.funambol.exchange.DataAccessException
     **/
    public void updateItems (Item[] localItems ,
                             String sourceURI  ,
                             long   principal  )
    throws DataAccessException {

         id.updateItems (localItems, sourceURI, principal);

    }

    /**
     * update local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public void upgradeItems (Item[] localItems ,
                              String sourceURI  ,
                              long   principal  )
    throws DataAccessException {

         id.deleteItems (localItems, sourceURI, principal);
         id.addItems    (localItems, sourceURI, principal);

    }

    /**
     * delete local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public void deleteItems(Item[] localItems ,
                            String sourceURI  ,
                            long   principal  )
    throws DataAccessException {

         id.deleteItems(localItems ,
                        sourceURI  ,
                        principal  );

    }

    /**
     * get Exchange Access Status
     *
     * @param exchangeServerName
     * @param userName
     * @param credentials
     *
     * @return array of find items
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public int getExchangeAccessStatus(String exchangeServerName ,
                                       String userName           ,
                                       String credentials        )
    throws DataAccessException {

        return id.getExchangeAccessStatus(exchangeServerName ,
                                          userName           ,
                                          credentials        );
    }

}
