/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.exchange.items.contact.dao;

import java.text.MessageFormat;

import java.util.ArrayList;
import java.util.List;
import java.util.TimeZone;

import com.funambol.foundation.pdi.contact.Address;
import com.funambol.foundation.pdi.contact.BusinessDetail;
import com.funambol.foundation.pdi.contact.Email;
import com.funambol.foundation.pdi.contact.Name;
import com.funambol.foundation.pdi.contact.PersonalDetail;
import com.funambol.foundation.pdi.contact.Phone;
import com.funambol.foundation.pdi.contact.Title;
import com.funambol.foundation.pdi.contact.WebPage;

import com.funambol.exchange.DataAccessException;
import com.funambol.exchange.httptransport.WebDavHttpTransport;
import com.funambol.exchange.items.contact.model.Contact;
import com.funambol.exchange.items.common.dao.ItemDAO;
import com.funambol.exchange.xml.XmlParseException;
import com.funambol.exchange.xml.XmlParser;
import com.funambol.exchange.util.DateTools;
import com.funambol.exchange.util.StringTools;

/*
 * This class implements methods to access contacts data
 * in exchange server datastore
 *
 * @version $Id: ContactDAO.java,v 1.3 2006/08/28 14:45:39 adeac Exp $
 *
 **/
public class ContactDAO extends ItemDAO {

    //---------------------------------------------------------------- Constants

    private static final String TAG_ANNIVERSARY
        = "weddinganniversary"                                             ;
    private static final String TAG_BIRTHDAY          = "bday"             ;
    private static final String TAG_CATEGORIES        = "keywordsutf8"     ;
    private static final String TAG_CHILDREN          = "childrensnames"   ;
    private static final String TAG_DEPARTMENT        = "department"       ;
    private static final String TAG_FILEAS            = "fileas"           ;
    public  static final String TAG_FIRSTNAME         = "givenname"        ;
    private static final String TAG_HOBBY             = "hobbies"          ;
    private static final String TAG_HOME_CITY         = "homecity"         ;
    private static final String TAG_HOME_COUNTRY      = "homecountry"      ;
    private static final String TAG_HOME_POST_OFFICE_BOX ="homepostofficebox";
    private static final String TAG_HOME_STATE        = "homestate"        ;
    private static final String TAG_HOME_POSTAL_CODE  = "homepostalcode"   ;
    private static final String TAG_HOME_STREET       = "homestreet"       ;
    private static final String TAG_IMPORTANCE        = "importance"       ;
    private static final String TAG_INITIALS          = "initials"         ;
    private static final String TAG_JOBTITLE          = "title"            ;
    private static final String TAG_LANGUAGE          = "language"         ;
    public  static final String TAG_LASTNAME          = "sn"        ;
    public  static final String TAG_MIDDLENAME        = "middlename"       ;
    private static final String TAG_MILEAGE           = "mileage"          ;
    private static final String TAG_SUFFIX            = "namesuffix"       ;
    private static final String TAG_NICKNAME          = "nickname"         ;
    private static final String TAG_OFFICE_LOCATION   = "roomnumber"       ;
    private static final String TAG_ORGANIZATION      = "organization"     ;
    private static final String TAG_ORGANIZATION_ID_NUMBER = "employeenumber";
    private static final String TAG_OTHER_CITY        = "othercity"        ;
    private static final String TAG_OTHER_COUNTRY     = "othercountry"     ;
    private static final String TAG_OTHER_POSTALCODE  = "otherpostalcode"  ;
    private static final String TAG_OTHER_POST_OFFICE_BOX = "otherpostofficebox";
    private static final String TAG_OTHER_STATE       = "otherstate"       ;
    private static final String TAG_OTHER_STREET      = "otherstreet"      ;
    private static final String TAG_SECRETARY         = "secretarycn"      ;
    private static final String TAG_SENSITIVITY       = "sensitivity"      ;
    private static final String TAG_SPOUSECN          = "spousecn"         ;
    private static final String TAG_TITLE             = "personaltitle"    ;
    private static final String TAG_WORK_CITY         = "city"             ;
    private static final String TAG_WORK_COUNTRY      = "country"          ;
    private static final String TAG_WORK_POSTALCODE   = "postalcode"       ;
    private static final String TAG_WORK_POST_OFFICE_BOX
        = "postofficebox"                                                  ;
    private static final String TAG_WORK_STATE        = "state"            ;
    private static final String TAG_WORK_STREET       = "street"           ;

    private static final String
        TAG_YOMI_COMPANY_NAME                         = "yomiorganization" ;
    private static final String TAG_YOMI_FIRST_NAME   = "yomilastname"     ;
    private static final String TAG_YOMI_LAST_NAME    = "yomifirstname"    ;

    private static final String TAG_CALL_BACK_PHONE   = "callbackphone"    ;
    private static final String
            TAG_FAC_SIMILE_TELEPHONE_NUMBER
            = "facsimiletelephonenumber"                                   ;
    private static final String TAG_HOME_FAX          = "homefax"          ;
    private static final String TAG_HOME_PHONE        = "homephone"        ;
    private static final String TAG_HOME_PHONE2       = "homephone2"       ;
    private static final String TAG_ISDN_NUMBER
        = "internationalisdnnumber"                                        ;
    private static final String TAG_MANAGER           = "manager"          ;
    private static final String TAG_MOBILE            = "mobile"           ;
    private static final String
           TAG_ORGANIZATION_MAIN_PHONE
           = "organizationmainphone"                                       ;
    private static final String TAG_OTHER_FAX         = "otherfax"         ;
    private static final String TAG_OTHER_MOBILE      = "othermobile"      ;
    private static final String TAG_OTHER_TELEPHONE
           = "othertelephone"                                              ;
    private static final String TAG_PAGER             = "pager"            ;
    private static final String TAG_PROFESSION        = "profession"       ;
    private static final String TAG_SECRETARY_PHONE   = "secretaryphone"   ;
    private static final String TAG_TELEPHONE_NUMBER  = "telephonenumber"  ;
    private static final String TAG_TELEPHONE_NUMBER2 = "telephonenumber2" ;
    private static final String TAG_TELEX_NUMBER      = "telexnumber"      ;
    private static final String TAG_RADIO_PHONE = "radioPhone";
    private static final String TAG_COMPUTER_NETWORK_NAME
        = "computernetworkname"                                            ;

    private static final String
        TAG_BUSINESS_HOME_PAGE
        = "businesshomepage"                                               ;
    private static final String
            TAG_PERSONAL_HOME_PAGE
        = "personalHomePage"                                               ;

    public  static final String TAG_EMAIL1            = "email1"           ;
    private static final String TAG_EMAIL2            = "email2"           ;
    private static final String TAG_EMAIL3            = "email3"           ;

    private static final String TAG_EMAIL1_ADDRESS_TYPE
        = "email1addrtype"                                                 ;
    private static final String TAG_EMAIL2_ADDRESS_TYPE
        = "email2addrtype"                                                 ;
    private static final String TAG_EMAIL3_ADDRESS_TYPE
        = "email3addrtype"                                                 ;

    private static final String TAG_NOTES = "notes";

    private static final String TAG_RESPONSE          = "a:response"       ;
    private static final String TAG_REPLUID           = "repluid"          ;
    private static final String TAG_LAST_MODIFIED     = "getlastmodified"  ;


    private static final String WEBDAV_MSG_SELECT_CONTACTS                   =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                          +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                  +
    "<D:sql>\n"                                                              +
    "Select "                                                                +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "            +
    "\"urn:schemas:contacts:weddinganniversary\" AS "
        + TAG_ANNIVERSARY                                                    + ", " +
    "\"urn:schemas:contacts:bday\" AS "             + TAG_BIRTHDAY           + ", " +
    "\"urn:schemas:contacts:childrensnames\" AS "   + TAG_CHILDREN           + ", " +
    "\"urn:schemas:contacts:fileas\" AS "           + TAG_FILEAS             + ", " +
    "\"urn:schemas:contacts:givenName\" AS "        + TAG_FIRSTNAME          + ", " +
    "\"http://schemas.microsoft.com/exchange/keywords-utf8\" AS "
                                                    + TAG_CATEGORIES         + ", " +
    "\"urn:schemas:contacts:hobbies\" AS "          + TAG_HOBBY              + ", " +
    "\"urn:schemas:httpmail:importance\" AS "       + TAG_IMPORTANCE         + ", " +
    "\"urn:schemas:contacts:initials\" AS "         + TAG_INITIALS           + ", " +
    "\"urn:schemas:contacts:sn\" AS "               + TAG_LASTNAME           + ", " +
    "\"urn:schemas:contacts:language\" AS "         + TAG_LANGUAGE           + ", " +
    "\"urn:schemas:contacts:title\" AS "            + TAG_JOBTITLE           + ", " +
    "\"urn:schemas:contacts:middlename\" AS "       + TAG_MIDDLENAME         + ", " +
    "\"urn:schemas:contacts:mileage\" AS "          + TAG_MILEAGE            + ", " +
    "\"urn:schemas:contacts:nickname\" AS "         + TAG_NICKNAME           + ", " +
    "\"urn:schemas:contacts:o\" AS "                + TAG_ORGANIZATION       + ", " +
    "\"urn:schemas:contacts:sensitivity\" AS "      + TAG_SENSITIVITY        + ", " +
    "\"urn:schemas:contacts:spousecn\" AS "         + TAG_SPOUSECN           + ", " +
    "\"urn:schemas:contacts:namesuffix\" AS "       + TAG_SUFFIX             + ", " +
    "\"urn:schemas:contacts:personaltitle\" AS "    + TAG_TITLE              + ", " +
    "\"http://schemas.microsoft.com/exchange/yomiorganization\" AS "
                                                    + TAG_YOMI_COMPANY_NAME  + ", " +
    "\"http://schemas.microsoft.com/exchange/yomifirstname\" AS "
                                                    + TAG_YOMI_FIRST_NAME    + ", " +
    "\"http://schemas.microsoft.com/exchange/yomilastname\" AS "
                                                    + TAG_YOMI_LAST_NAME     + ", " +
    "\"urn:schemas:contacts:homeStreet\" AS "       + TAG_HOME_STREET        + ", " +
    "\"urn:schemas:contacts:homeCity\" AS "         + TAG_HOME_CITY          + ", " +
    "\"urn:schemas:contacts:homeState\" AS "        + TAG_HOME_STATE         + ", " +
    "\"urn:schemas:contacts:homePostalCode\" AS "   + TAG_HOME_POSTAL_CODE   + ", " +
    "\"urn:schemas:contacts:homepostofficebox\" AS "+
        TAG_HOME_POST_OFFICE_BOX                                             + ", " +
    "\"urn:schemas:contacts:homeCountry\" AS "      + TAG_HOME_COUNTRY       + ", " +

    "\"urn:schemas:contacts:street\" AS "           + TAG_WORK_STREET        + ", " +
    "\"urn:schemas:contacts:l\" AS "                + TAG_WORK_CITY          + ", " +
    "\"urn:schemas:contacts:st\" AS "               + TAG_WORK_STATE         + ", " +
    "\"urn:schemas:contacts:postalcode\" AS "       + TAG_WORK_POSTALCODE    + ", " +
    "\"urn:schemas:contacts:postofficebox\" AS "    +
        TAG_WORK_POST_OFFICE_BOX                                             + ", " +
    "\"urn:schemas:contacts:co\" AS "               + TAG_WORK_COUNTRY       + ", " +

    "\"urn:schemas:contacts:otherstreet\" AS "      + TAG_OTHER_STREET       + ", " +
    "\"urn:schemas:contacts:othercity\" AS "        + TAG_OTHER_CITY         + ", " +
    "\"urn:schemas:contacts:otherstate\" AS "       + TAG_OTHER_STATE        + ", " +
    "\"urn:schemas:contacts:otherpostalcode\" AS "  + TAG_OTHER_POSTALCODE   + ", " +
    "\"urn:schemas:contacts:otherpostofficebox\" AS "
        + TAG_OTHER_POST_OFFICE_BOX                                          + ", " +
    "\"urn:schemas:contacts:othercountry\" AS "     + TAG_OTHER_COUNTRY      + ", " +

    "\"urn:schemas:contacts:callbackphone\" AS "    + TAG_CALL_BACK_PHONE    + ", " +
    "\"urn:schemas:contacts:facsimiletelephonenumber\" "                     +
            "AS "                                   +
              TAG_FAC_SIMILE_TELEPHONE_NUMBER                                + ", " +
    "\"urn:schemas:contacts:homefax\" AS "          + TAG_HOME_FAX           + ", " +
    "\"urn:schemas:contacts:homePhone\" AS "        + TAG_HOME_PHONE         + ", " +
    "\"urn:schemas:contacts:homephone2\" AS "       + TAG_HOME_PHONE2        + ", " +
    "\"urn:schemas:contacts:mobile\" AS "           + TAG_MOBILE             + ", " +
    "\"urn:schemas:contacts:internationalisdnnumber\" AS " +
             TAG_ISDN_NUMBER                                                 + ", " +
    "\"urn:schemas:contacts:organizationmainphone\" "                        +
            "AS "                                   +
            TAG_ORGANIZATION_MAIN_PHONE                                      + ", " +
    "\"urn:schemas:contacts:otherfax\" AS "         + TAG_OTHER_FAX          + ", " +
    "\"urn:schemas:contacts:othermobile\" AS "      + TAG_OTHER_MOBILE       + ", " +
    "\"urn:schemas:contacts:otherTelephone\" AS "   + TAG_OTHER_TELEPHONE    + ", " +
    "\"urn:schemas:contacts:pager\" AS "            + TAG_PAGER              + ", " +
    "\"urn:schemas:contacts:secretaryphone\" AS "   + TAG_SECRETARY_PHONE    + ", " +
    "\"urn:schemas:contacts:telephoneNumber\" AS "  + TAG_TELEPHONE_NUMBER   + ", " +
    "\"urn:schemas:contacts:telephonenumber2\" AS " + TAG_TELEPHONE_NUMBER2  + ", " +
    "\"urn:schemas:contacts:telexnumber\" AS "      + TAG_TELEX_NUMBER       + ", " +
    "\"http://schemas.microsoft.com/mapi/proptag/0x3A1D001F\" AS " + TAG_RADIO_PHONE + ", " +

    "\"urn:schemas:contacts:businesshomepage\" AS " + TAG_BUSINESS_HOME_PAGE + ", " +
    "\"urn:schemas:contacts:personalHomePage\" AS " + TAG_PERSONAL_HOME_PAGE + ", " +

    "\"urn:schemas:contacts:department\" AS "       + TAG_DEPARTMENT         + ", " +
    "\"urn:schemas:contacts:roomnumber\" AS "       + TAG_OFFICE_LOCATION    + ", " +
    "\"urn:schemas:contacts:employeenumber\" AS "   +
        TAG_ORGANIZATION_ID_NUMBER                                           + ", " +
    "\"urn:schemas:contacts:profession\" AS "       + TAG_PROFESSION         + ", " +
    "\"urn:schemas:contacts:manager\" AS "          + TAG_MANAGER            + ", " +
    "\"urn:schemas:contacts:secretarycn\" AS "      + TAG_SECRETARY          + ", " +

    "\"urn:schemas:contacts:email1\" AS "           + TAG_EMAIL1             + ", " +
    "\"urn:schemas:contacts:email2\" AS "           + TAG_EMAIL2             + ", " +
    "\"urn:schemas:contacts:email3\" AS "           + TAG_EMAIL3             + ", " +
    "\"urn:schemas:contacts:email1addrtype\" AS "   +
        TAG_EMAIL1_ADDRESS_TYPE                                              + ", " +
    "\"urn:schemas:contacts:email2addrtype\" AS "   +
        TAG_EMAIL2_ADDRESS_TYPE                                              + ", " +
    "\"urn:schemas:contacts:email3addrtype\" AS "   +
        TAG_EMAIL3_ADDRESS_TYPE                                              + ", " +

    "\"urn:schemas:httpmail:textdescription\" AS "+TAG_NOTES+","+

    "\"urn:schemas:contacts:computernetworkname\" AS "
        + TAG_COMPUTER_NETWORK_NAME                                          + ", " +
    "\"DAV:displayname\", "                                                  +
    "\"DAV:getlastmodified\" AS getlastmodified, "                           +
    "\"DAV:href\" AS href, "                                                 +
    "\"DAV:isfolder\" AS isfolder "                                          +
    "FROM \"/{0}/{1}/{2}\"  "                                                +
    "{3} "                                                                   +
    "</D:sql>\n"                                                             +
    "</D:searchrequest>"                                                     ;


    private static final String WEBDAV_MSG_SELECT_CONTACTS_FOR_TWIN          =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                          +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                  +
    "<D:sql>\n"                                                              +
    "Select "                                                                +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "            +
    "\"DAV:getlastmodified\" AS getlastmodified, "                           +
    "\"DAV:creationdate\" AS creationdate, "                                 +
    "\"DAV:href\" AS href, "                                                 +
    "\"urn:schemas:contacts:givenName\" AS " + TAG_FIRSTNAME                 +
                                                    ", " +
    "\"urn:schemas:contacts:sn\" AS "        + TAG_LASTNAME                  +
                                                    ", " +
    "\"DAV:isfolder\" AS isfolder "                                          +
    "FROM \"/{0}/{1}/{2}\"  "                                                +
    "{3}"                                                                    +
    "</D:sql>\n"                                                             +
    "</D:searchrequest>"                                                     ;

    //private static final String
    //    FIELD_PREFIX                  = "urn:schemas:contacts:"    ;

    private static final String
        WEBDAV_INIT_EX                = "EX:"                      ;
    //private static final String
    //    WEBDAV_INIT_HG                = "HG:"                      ;
    private static final String
        WEBDAV_INIT_HM                = "HM:"                      ;
    private static final String
        WEBDAV_INIT_HN                = "HN:"                      ;
    private static final String
        WEBDAV_INIT_HO                = "HO:"                      ;

    private static final String
        WEBDAV_INIT_MAPI              = "MAPI:"                    ;

    private static final String WEBDAV_INIT_MAPI_PROPTAG = "MAPIPT:";

    private static final String
        WEBDAV_TAG_ANNIVERSARY        = WEBDAV_INIT_HM             +
                                        "weddinganniversary"       ;
    private static final String
        WEBDAV_TAG_ASSISTANT          = WEBDAV_INIT_HM             +
                                        "secretarycn"              ;
    private static final String
        WEBDAV_TAG_BIRTHDAY           = WEBDAV_INIT_HM             +
                                        "bday"                     ;
    private static final String
        WEBDAV_TAG_CATEGORIES         = WEBDAV_INIT_EX             +
                                        "keywords-utf8"            ;
    private static final String
        WEBDAV_TAG_CHILDREN           = WEBDAV_INIT_HM             +
                                        "childrensnames"           ;
    private static final String
        WEBDAV_TAG_CN                 = WEBDAV_INIT_HM             +
                                        "cn"                       ;
    private static final String
        WEBDAV_TAG_COMPANY            = WEBDAV_INIT_HM             +
                                        "o"                        ;
    private static final String
        WEBDAV_TAG_DEPARTMENT         = WEBDAV_INIT_HM             +
                                        "department"               ;
    private static final String
        WEBDAV_TAG_FILE_AS            = WEBDAV_INIT_HM             +
                                        "fileas"                   ;
    private static final String
        WEBDAV_TAG_FIRST_NAME         = WEBDAV_INIT_HM             +
                                        "givenName"                ;
    private static final String
        WEBDAV_TAG_HOBBY              = WEBDAV_INIT_HM             +
                                        "hobbies"                  ;
    private static final String
        WEBDAV_TAG_HOME_CITY          = WEBDAV_INIT_HM             +
                                        "homeCity"                 ;
    private static final String
        WEBDAV_TAG_HOME_COUNTRY       = WEBDAV_INIT_HM             +
                                        "homeCountry"              ;
    private static final String
        WEBDAV_TAG_HOME_POSTALCODE    = WEBDAV_INIT_HM             +
                                        "homePostalCode"           ;
    private static final String
        WEBDAVTAG_HOME_POST_OFFICE_BOX
                                      = WEBDAV_INIT_HM             +
                                        "homepostofficebox"        ;
    private static final String
        WEBDAV_TAG_HOME_STATE         = WEBDAV_INIT_HM             +
                                        "homeState"                ;
    private static final String
        WEBDAV_TAG_HOME_STREET        = WEBDAV_INIT_HM             +
                                        "homeStreet"               ;
    private static final String
        WEBDAV_TAG_IMPORTANCE         = WEBDAV_INIT_HN             +
                                        "importance"               ;
    private static final String
        WEBDAV_TAG_INITIALS           = WEBDAV_INIT_HM             +
                                        "initials"                 ;
    private static final String
        WEBDAV_TAG_JOB_TITLE          = WEBDAV_INIT_HM             +
                                        "title"                    ;
    private static final String
        WEBDAV_TAG_LANGUAGE          =  WEBDAV_INIT_HM             +
                                        "language";
    private static final String
        WEBDAV_TAG_LAST_NAME          = WEBDAV_INIT_HM             +
                                        "sn"                       ;
    private static final String
        WEBDAV_TAG_MANAGER            = WEBDAV_INIT_HM             +
                                        "manager"                  ;
    private static final String
        WEBDAV_TAG_MIDDLE_NAME        = WEBDAV_INIT_HM             +
                                        "middlename"               ;
    private static final String
        WEBDAV_TAG_MILEAGE            = WEBDAV_INIT_EX             +
                                        "mileage"                  ;
    private static final String
        WEBDAV_TAG_NICKNAME           = WEBDAV_INIT_HM             +
                                        "nickname"                 ;
    private static final String
        WEBDAV_TAG_OFFICE_LOCATION    = WEBDAV_INIT_HM             +
                                        "roomnumber"               ;
    private static final String
        WEBDAV_TAG_OTHER_CITY         = WEBDAV_INIT_HM             +
                                        "othercity"                ;
    private static final String
        WEBDAV_TAG_OTHER_COUNTRY      = WEBDAV_INIT_HM             +
                                        "othercountry"             ;
    private static final String
        WEBDAV_TAG_OTHER_POSTALCODE   = WEBDAV_INIT_HM             +
                                        "otherpostalcode"          ;
    private static final String
        WEBDAV_TAG_OTHER_POST_OFFICE_BOX
                                      = WEBDAV_INIT_HM             +
                                        "otherpostofficebox"       ;

    private static final String
        WEBDAV_TAG_OTHER_STATE        = WEBDAV_INIT_HM             +
                                        "otherstate"               ;
    private static final String
        WEBDAV_TAG_OTHER_STREET       = WEBDAV_INIT_HM             +
                                        "otherstreet"              ;
    //private static final String
    //    WEBDAV_TAG_ORGANIZATION_ID_NUMBER
    //                                  = WEBDAV_INIT_HM             +
    //                                    "employeenumber"           ;
    private static final String
        WEBDAV_TAG_PROFESSION         = WEBDAV_INIT_HM             +
                                        "profession"               ;
    private static final String
        WEBDAV_TAG_SENSITIVITY        = WEBDAV_INIT_MAPI           +
                                        "sensitivity"              ;
    private static final String
        WEBDAV_TAG_SPOUSE             = WEBDAV_INIT_HM             +
                                        "spousecn"                 ;
    private static final String
        WEBDAV_TAG_SUBJECT1           = WEBDAV_INIT_HN             +
                                        "subject"                  ;
    private static final String
        WEBDAV_TAG_SUBJECT2           = WEBDAV_INIT_HO             +
                                        "subject"                  ;
    private static final String
        WEBDAV_TAG_SUFFIX             = WEBDAV_INIT_HM             +
                                        "namesuffix"               ;
    private static final String
        WEBDAV_TAG_TITLE              = WEBDAV_INIT_HM             +
                                        "personaltitle"            ;
    private static final String
        WEBDAV_TAG_WORK_CITY          = WEBDAV_INIT_HM             +
                                        "l"                        ;
    private static final String
        WEBDAV_TAG_WORK_COUNTRY       = WEBDAV_INIT_HM             +
                                        "co"                       ;
    private static final String
        WEBDAV_TAG_WORK_POSTALCODE    = WEBDAV_INIT_HM             +
                                        "postalcode"               ;
    private static final String
        WEBDAV_TAG_WORK_POST_OFFICE_BOX = WEBDAV_INIT_HM           +
                                        "postofficebox"            ;
    private static final String
        WEBDAV_TAG_WORK_STATE         = WEBDAV_INIT_HM             +
                                        "st"                       ;
    private static final String
        WEBDAV_TAG_WORK_STREET        = WEBDAV_INIT_HM             +
                                        "street"                   ;

    //private static final String
    //    WEBDAV_TAG_YOMI_COMAPANY_NAME = WEBDAV_INIT_EX             +
    //                                    "yomiorganization"         ;
    //private static final String
    //    WEBDAV_TAG_YOMI_FIRST_NAME    = WEBDAV_INIT_EX             +
    //                                    "yomifirstname"            ;
    //private static final String
    //    WEBDAV_TAG_YOMI_LAST_NAME     = WEBDAV_INIT_EX             +
    //                                    "yomilastname"             ;
    private static final String
        WEBDAV_TAG_ASSISTANT_PHONE    = WEBDAV_INIT_HM             +
                                        "secretaryphone"           ;
    private static final String
        WEBDAV_TAG_CALLBACK_PHONE     = WEBDAV_INIT_HM             +
                                        "callbackphone"            ;
    private static final String
        WEBDAV_TAG_COMPANY_PHONE      = WEBDAV_INIT_HM             +
                                        "organizationmainphone"    ;
    private static final String
        WEBDAV_TAG_HOME_FAX           = WEBDAV_INIT_HM             +
                                        "homefax"                  ;
    private static final String
        WEBDAV_TAG_HOME_OTHER_PHONE   = WEBDAV_INIT_HM             +
                                        "homephone2"               ;
    private static final String
        WEBDAV_TAG_HOME_PHONE         = WEBDAV_INIT_HM             +
                                        "homePhone"                ;
    private static final String
        WEBDAV_TAG_MOBILE_OTHER_PHONE = WEBDAV_INIT_HM             +
                                        "othermobile"              ;
    private static final String
        WEBDAV_TAG_MOBILE_PHONE       = WEBDAV_INIT_HM             +
                                        "mobile"                   ;
    private static final String
        WEBDAV_TAG_OTHER_FAX          = WEBDAV_INIT_HM             +
                                        "otherfax"                 ;
    private static final String
        WEBDAV_TAG_OTHER_PHONE        = WEBDAV_INIT_HM             +
                                        "otherTelephone"           ;
    private static final String
        WEBDAV_TAG_PAGER              = WEBDAV_INIT_HM             +
                                        "pager"                    ;
    private static final String WEBDAV_TAG_RADIO_PHONE =
            WEBDAV_INIT_MAPI_PROPTAG + "0x3A1D001F";

    private static final String
        WEBDAV_TAG_WORK_FAX           = WEBDAV_INIT_HM             +
                                        "facsimiletelephonenumber" ;
    private static final String
        WEBDAV_TAG_WORK_OTHER_PHONE   = WEBDAV_INIT_HM             +
                                        "telephonenumber2"         ;
    private static final String
        WEBDAV_TAG_WORK_PHONE         = WEBDAV_INIT_HM             +
                                        "telephoneNumber"          ;

    private static final String
        WEBDAV_TAG_BUSINESS_HOME_PAGE = WEBDAV_INIT_HM             +
                                        "businesshomepage"         ;
    private static final String
        WEBDAV_TAG_PERSONAL_HOME_PAGE = WEBDAV_INIT_HM             +
                                        "personalHomePage"         ;
    private static final String
        WEBDAV_TAG_EMAIL1             = WEBDAV_INIT_HM             +
                                        "email1"                   ;
    private static final String
        WEBDAV_TAG_EMAIL2             = WEBDAV_INIT_HM             +
                                        "email2"                   ;
    private static final String
        WEBDAV_TAG_EMAIL3             = WEBDAV_INIT_HM             +
                                        "email3"                   ;
   
    private static final String WEBDAV_TAG_NOTES = WEBDAV_INIT_HN + "textdescription";

    private static final String
        FIELD_TYPE_ASSISTANT_NUMBER                     =
        "AssistantTelephoneNumber"                      ;
    private static final String
        FIELD_TYPE_BUSINESS_FAX_NUMBER                  =
        "BusinessFaxNumber"                             ;
    private static final String
        FIELD_TYPE_BUSINESS_OTHER_TELEPHONE_NUMBER      =
        "Business2TelephoneNumber"                      ;
    private static final String
        FIELD_TYPE_BUSINESS_TELEPHONE_NUMBER            =
        "BusinessTelephoneNumber"                       ;
    private static final String
        FIELD_TYPE_CALLBACK_NUMBER                      =
        "CallbackTelephoneNumber"                       ;
    private static final String
        FIELD_TYPE_CAR_TELEPHONE_NUMBER                 =
        "CarTelephoneNumber"                            ;
    private static final String
        FIELD_TYPE_COMPANY_MAIN_TELEPHONE_NUMBER        =
        "CompanyMainTelephoneNumber"                     ;
    public static final String
        FIELD_TYPE_EMAIL1_ADDRESS                       =
        "Email1Address"                                 ;
    public static final String
        FIELD_TYPE_EMAIL2_ADDRESS                       =
        "Email2Address"                                 ;
    private static final String
        FIELD_TYPE_EMAIL3_ADDRESS                       =
        "Email3Address"                                 ;
    private static final String
        FIELD_TYPE_JOB_TITLE                            =
        "JobTitle"                                      ;

    private static final String
        FIELD_TYPE_PERSONAL_HOME_PAGE                   =
        "HomeWebPage"                                   ;
    private static final String
        FIELD_TYPE_BUSINESS_HOME_PAGE                   =
        "WebPage"                                       ;
    private static final String
        FIELD_TYPE_HOME_TELEPHONE_NUMBER                =
        "HomeTelephoneNumber"                           ;
    private static final String
        FIELD_TYPE_HOME_OTHER_TELEPHONE_NUMBER          =
        "Home2TelephoneNumber"                          ;
    private static final String
        FIELD_TYPE_HOME_FAX_NUMBER                      =
        "HomeFaxNumber"                                 ;
    private static final String
        FIELD_TYPE_MOBILE_TELEPHONE_NUMBER              =
        "MobileTelephoneNumber"                         ;
    private static final String
        FIELD_TYPE_OTHER_TELEPHONE_NUMBER               =
        "OtherTelephoneNumber"                          ;
    private static final String
        FIELD_TYPE_OTHER_FAX_NUMBER                     =
        "OtherFaxNumber"                                ;
    private static final String
        FIELD_TYPE_PAGER_NUMBER                         =
        "PagerNumber"                                   ;
    private static final String FIELD_TYPE_RADIO_PHONE = "RadioTelephoneNumber";
    private static final String FIELD_TYPE_NOTES = "Body";

    //------------------------------------------------------------- Private data

    private WebDavHttpTransport  webDavHttp          = null ;

    private TimeZone deviceTimeZone = null;

    //-------------------------------------------------------------- Costructors

    public ContactDAO(String  host           ,
                      int     port           ,
                      String  exchangeServer,
                      TimeZone deviceTimZone)
        throws DataAccessException {

        this.webDavHttp         = new WebDavHttpTransport(host           ,
                                                          port           ,
                                                          exchangeServer );
        this.deviceTimeZone = deviceTimZone;
    }

    //----------------------------------------------------------- Public methods

    /**
     * insert / update contact<br>
     *
     * @param c
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new / updated contact
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Contact setContact(Contact c              ,
                              String  username       ,
                              String  credentials    ,
                              String  exchangeFolder )
    throws DataAccessException {

        PersonalDetail    personalDetail         ;
        BusinessDetail    businessDetail         ;
        Address           homeAddressBook        = null ;
        Address           workAddressBook        = null ;
        Address           otherAddressBook       = null ;

        Name              name                   ;
        Phone             phone                  ;
        Email             email                  ;
        WebPage           webPage                ;

        List              emails                 = null ;
        List              webPagesPersonal       = null ;
        List              emailsBusiness         = null ;
        List              phonesPersonal         = null ;
        List              phonesBusiness         = null ;

        String            response               ;

        String            id                     ;

        StringBuffer      webDavContactMsg       ;
        String            webDavHeaderMsg        ;

        String            anniversary            = null ;
        String            birthday               = null ;
        String            categories             = null ;
        String            children               = null ;
        String            firstName              = null ;
        String            hobby                  = null ;
        Short             importance             ;
        String            initials               = null ;
        String            language               ;
        String            lastName               = null ;
        String            middleName             = null ;
        String            mileage                ;
        String            jobTitle               ;
        String            organization           = null ;
        Short             sensitivity            ;
        String            suffix                 = null ;
        String            title                  = null ;
        String            homeStreet             = null ;
        String            homeCity               = null ;
        String            homeState              = null ;
        String            homePostalCode         = null ;
        String            homePostOfficeBox      = null ;
        String            homeCountry            = null ;
        String            officeStreet           = null ;
        String            officeCity             = null ;
        String            officeState            = null ;
        String            officeCountry          = null ;
        String            officePostalCode       = null ;
        String            officePostOfficeBox    = null ;
        String            officeLocation         = null ;
        String            otherStreet            = null ;
        String            otherCity              = null ;
        String            otherState             = null ;
        String            otherPostalCode        = null ;
        String            otherPostOfficeBox     = null ;
        String            otherCountry           = null ;
        String            officePhone            = null ;
        String            homePhone              = null ;
        String            mobilePhone            = null ;
        String            assistantPhone         = null ;
        String            officeOtherPhone       = null ;
        String            officeFax              = null ;
        String            callBackPhone          = null ;
        String            mobileOtherPhone       = null ;
        String            organizationPhone      = null ;
        String            homeOtherPhone         = null ;
        String            homeFax                = null ;
        String            otherPhone             = null ;
        String            otherFax               = null ;
        String            pager                  = null ;
        String radioPhone = null;
        String            personalHomePage       = null ;
        String            businessHomePage       = null ;
        String            department             = null ;
        String            profession             = null ;
        String            nickName               = null ;
        String            spouse                 = null ;
        String            manager                = null ;
        String            assistant              = null ;
        String            emailAddress1          = null ;
        String            emailAddress2          = null ;
        String            emailAddress3          = null ;
        String            notes                  = null;

        String            phoneType              ;
        String            webPageType            ;

        String            value                  ;

        String            server                 ;
        String            resource               ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        personalDetail        = c.getPersonalDetail                 ()    ;
        businessDetail        = c.getBusinessDetail                 ()    ;
        name                  = c.getName                           ()    ;

        importance            = c.getImportance                     ()    ;
        sensitivity           = c.getSensitivity                    ()    ;
        mileage               = c.getMileage                        ()    ;
        language              = c.getLanguages                      ()    ;

        if (c.getCategories() != null) {
            categories = c.getCategories().getPropertyValueAsString();
        }


        if (personalDetail != null) {
            homeAddressBook       = personalDetail.getAddress       ()    ;
            otherAddressBook      = personalDetail.getOtherAddress  ()    ;
            emails                = personalDetail.getEmails        ()    ;
            phonesPersonal        = personalDetail.getPhones        ()    ;
            spouse                = personalDetail.getSpouse        ()    ;
            children              = personalDetail.getChildren      ()    ;
            birthday              = personalDetail.getBirthday      ()    ;
            anniversary           = personalDetail.getAnniversary   ()    ;
            hobby                 = personalDetail.getHobbies       ()    ;
            webPagesPersonal      = personalDetail.getWebPages      ()    ;
        }
        if (businessDetail != null) {
            workAddressBook       = businessDetail.getAddress       ()    ;
            phonesBusiness        = businessDetail.getPhones        ()    ;
            emailsBusiness        = businessDetail.getEmails        ()    ;
        }

        if (businessDetail != null) {
            assistant      = businessDetail.getAssistant            ()    ;
            manager        = businessDetail.getManager              ()    ;
            officeLocation = businessDetail.getOfficeLocation       ()    ;

            if (businessDetail.getCompany() != null) {
                organization     = businessDetail.getCompany    ().
                                            getPropertyValueAsString()    ;
            }
            if (businessDetail.getDepartment() != null) {
                department  = businessDetail.getDepartment ().
                                            getPropertyValueAsString()     ;
            }
            if (businessDetail.getRole() != null) {
                profession        = businessDetail.getRole       ().
                                            getPropertyValueAsString()     ;
            }
        }

        if (name != null) {
            if (name.getFirstName() != null) {
                firstName   = name.getFirstName        ().
                                        getPropertyValueAsString()         ;
            }
            if (name.getMiddleName() != null) {
                middleName  = name.getMiddleName       ().
                                        getPropertyValueAsString()         ;
            }
            if (name.getLastName() != null) {
                lastName    = name.getLastName         ().
                                        getPropertyValueAsString()         ;
            }
            //if (name.getDisplayName() != null) {
            //    displayName = name.getDisplayName      ().
            //                            getPropertyValueAsString()       ;
            //}
            if (name.getNickname() != null) {
                nickName    = name.getNickname         ().
                                        getPropertyValueAsString()         ;
            }

            if (name.getInitials() != null) {
                initials    = name.getInitials         ().
                                        getPropertyValueAsString()         ;
            }

            if (name.getSuffix() != null) {
                suffix      = name.getSuffix           ().
                                        getPropertyValueAsString()         ;
            }
            if (name.getSalutation() != null) {
                title  = name.getSalutation       ().
                                        getPropertyValueAsString()         ;
            }

        }

        if (businessDetail.getTitles() !=null &&
            businessDetail.getTitles().size() > 0 &&
            (businessDetail.getTitles().get(0)) != null) {
            jobTitle = ((Title) businessDetail.getTitles().get(0)).
                                                getPropertyValueAsString();
        } else {
            jobTitle = null;
        }

        if (emails != null) {
            for (int i=0, l = emails.size(); i < l; i++) {
                email = (Email) emails.get(i);
                if ((FIELD_TYPE_EMAIL1_ADDRESS).equals(email.getEmailType())) {
                    emailAddress1 = email.getPropertyValueAsString();

                } else if ((FIELD_TYPE_EMAIL2_ADDRESS).
                            equals(email.getEmailType())) {
                    emailAddress2 = email.getPropertyValueAsString();
                } else if ((FIELD_TYPE_EMAIL3_ADDRESS).
                            equals(email.getEmailType())) {
                    emailAddress3 = email.getPropertyValueAsString();
                }
            }
        }

        if (emailsBusiness != null) {
            java.util.Iterator it = emailsBusiness.iterator();
            while (it.hasNext())
            {
                email = (Email) it.next();
                if (emailAddress1==null && (FIELD_TYPE_EMAIL1_ADDRESS).equals(email.getEmailType()))
                {
                    emailAddress1 = email.getPropertyValueAsString();
                    continue;
                }
                if (emailAddress2==null &&
                           (FIELD_TYPE_EMAIL2_ADDRESS).
                            equals(email.getEmailType()))
                {
                    emailAddress2 = email.getPropertyValueAsString();
                    continue;
                }
                if (emailAddress3==null && (FIELD_TYPE_EMAIL3_ADDRESS). equals(email.getEmailType()))
                {
                    emailAddress3 = email.getPropertyValueAsString();
                }
            }
        }

        //
        // phones
        //
        if (phonesPersonal != null) {

            for (int i=0, l = phonesPersonal.size(); i < l; i++) {

                phone = (Phone) phonesPersonal.get(i);

                phoneType = phone.getPhoneType();

                if ((FIELD_TYPE_HOME_TELEPHONE_NUMBER).
                                equals(phoneType))           {
                    homePhone = phone.getPropertyValueAsString();
                } if ((FIELD_TYPE_HOME_OTHER_TELEPHONE_NUMBER).
                                equals(phoneType))           {
                    homeOtherPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_HOME_FAX_NUMBER).
                                equals(phoneType))           {
                    homeFax = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_MOBILE_TELEPHONE_NUMBER).
                                equals(phoneType))           {
                    mobilePhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_CAR_TELEPHONE_NUMBER).
                                equals(phoneType))           {
                    mobileOtherPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_OTHER_TELEPHONE_NUMBER).
                                equals(phoneType))           {
                    otherPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_OTHER_FAX_NUMBER).
                                equals(phoneType))           {
                    otherFax = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_RADIO_PHONE).equals(phoneType)) {
                    radioPhone = phone.getPropertyValueAsString();
                } else                                       {
                    //
                    // no save uknown property
                    //
                    //continue;
                }

            }

        }

        //
        // phones business
        //
        if (phonesBusiness != null) {

            for (int i=0, l = phonesBusiness.size(); i < l; i++) {

                phone = (Phone) phonesBusiness.get(i);

                phoneType = phone.getPhoneType();

                if ((FIELD_TYPE_BUSINESS_TELEPHONE_NUMBER).
                            equals(phoneType))                    {
                    officePhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_BUSINESS_OTHER_TELEPHONE_NUMBER).
                            equals(phoneType))                    {
                    officeOtherPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_BUSINESS_FAX_NUMBER).
                            equals(phoneType))                    {
                    officeFax = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_COMPANY_MAIN_TELEPHONE_NUMBER).
                            equals(phoneType))                    {
                    organizationPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_PAGER_NUMBER).
                            equals(phoneType))                    {
                    pager = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_ASSISTANT_NUMBER).
                            equals(phoneType))                    {
                    assistantPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_CALLBACK_NUMBER).
                            equals(phoneType))                    {
                    callBackPhone = phone.getPropertyValueAsString();
                } else                                            {
                    //
                    // no save uknown property
                    //
                }

            }

        }

        //
        // webPages personal
        //

        if (webPagesPersonal != null) {

            for (int i=0, l = webPagesPersonal.size(); i < l; i++)     {

                webPage = (WebPage) webPagesPersonal.get(i);

                webPageType = webPage.getWebPageType();

                if ((FIELD_TYPE_PERSONAL_HOME_PAGE).equals(webPageType))         {
                    personalHomePage = webPage.getPropertyValueAsString();
                } else  if ((FIELD_TYPE_BUSINESS_HOME_PAGE).equals(webPageType)) {
                    businessHomePage = webPage.getPropertyValueAsString();
                 }                                                               {
                    //
                    // no save uknown property
                    //

                }

            }

        }

        if (homeAddressBook != null) {

            if (homeAddressBook.getStreet() != null) {
                homeStreet = homeAddressBook.getStreet().
                                            getPropertyValueAsString();
            }
            if (homeAddressBook.getCity() != null) {
                homeCity = homeAddressBook.getCity().
                                            getPropertyValueAsString();
            }
            if (homeAddressBook.getPostalCode() != null) {
                homePostalCode = homeAddressBook.getPostalCode().
                                            getPropertyValueAsString();
            }
            if (homeAddressBook.getPostOfficeAddress() != null) {
                homePostOfficeBox = homeAddressBook.
                                        getPostOfficeAddress().
                                                getPropertyValueAsString();
            }
            if (homeAddressBook.getState() != null) {
                homeState = homeAddressBook.getState().
                                            getPropertyValueAsString();
            }
            if (homeAddressBook.getCountry()!= null) {
                homeCountry = homeAddressBook.getCountry().
                                            getPropertyValueAsString();
            }

        }

        if (otherAddressBook != null) {

            if (otherAddressBook.getStreet() != null) {
                otherStreet = otherAddressBook.getStreet().
                                        getPropertyValueAsString();
            }
            if (otherAddressBook.getCity() != null) {
                otherCity = otherAddressBook.getCity().
                                        getPropertyValueAsString();
            }
            if (otherAddressBook.getPostalCode() != null) {
                otherPostalCode = otherAddressBook.getPostalCode().
                                        getPropertyValueAsString();
            }
            if (otherAddressBook.getPostOfficeAddress() != null) {
                otherPostOfficeBox = otherAddressBook.
                                        getPostOfficeAddress().
                                                getPropertyValueAsString();
            }
            if (otherAddressBook.getState() != null) {
                otherState = otherAddressBook.getState().
                                        getPropertyValueAsString();
            }
            if (homeAddressBook.getCountry()!= null) {
                otherCountry = otherAddressBook.getCountry().
                                        getPropertyValueAsString();
            }

        }

        if (workAddressBook != null) {

            if (workAddressBook.getStreet() != null) {
                officeStreet = workAddressBook.getStreet().
                                    getPropertyValueAsString();
            }
            if (workAddressBook.getCity() != null) {
                officeCity = workAddressBook.getCity().
                                    getPropertyValueAsString();
            }
            if (workAddressBook.getPostalCode() != null) {
                officePostalCode = workAddressBook.getPostalCode().
                                    getPropertyValueAsString();
            }
            if (workAddressBook.getPostOfficeAddress() != null) {
                officePostOfficeBox = workAddressBook.
                                        getPostOfficeAddress().
                                                getPropertyValueAsString();
            }
            if (workAddressBook.getState() != null) {
                officeState = workAddressBook.getState().
                                    getPropertyValueAsString();
            }
            if (workAddressBook.getCountry()!= null) {
                officeCountry = workAddressBook.getCountry().
                                    getPropertyValueAsString();
            }

            if (c.getNotes()!=null && c.getNotes().size()>0 && c.getNotes().get(0)!=null)
            {
                com.funambol.foundation.pdi.contact.Note note =
                    (com.funambol.foundation.pdi.contact.Note) c.getNotes().get(0);
                notes = note.getPropertyValueAsString();
            }

        }

        webDavContactMsg = new StringBuffer(INITIAL_SIZE_SBUFFER);

        webDavContactMsg.
             append ("<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                 )
            .append("<D:propertyupdate xmlns:D=\"DAV:\" "                            )
            .append("xmlns:EX=\"http://schemas.microsoft.com/exchange/\" "           )
            .append("xmlns:HM=\"urn:schemas:contacts:\" "                            )
            .append("xmlns:HG=\"urn:schemas:calendar:\" "                            )
            .append("xmlns:HN=\"urn:schemas:httpmail:\" "                            )
            .append("xmlns:x=\"xml:\" "                                              )
            .append("xmlns:MAPI=\"http://schemas.microsoft.com/mapi/\" "             )
            .append("xmlns:MAPIPT=\"http://schemas.microsoft.com/mapi/proptag/\" ")
            .append("xmlns:HO=\"urn:schemas:mailheader:\">\n"                        )
            .append("<D:set>\n"                                                      )
            .append("<D:prop>\n"                                                     )
            .append("<D:contentclass>urn:content-classes:person</D:contentclass>\n"  )
            .append("<EX:outlookmessageclass>IPM.Contact</EX:outlookmessageclass>\n" ) ;


        firstName   = StringTools.escapeXml(firstName  );
        lastName    = StringTools.escapeXml(lastName   );
        middleName  = StringTools.escapeXml(middleName );
        initials    = StringTools.escapeXml(initials   );

        //
        // full name variables
        //
        String fnTitle      = title      ;
        String fnFirstName  = firstName  ;
        String fnMiddleName = middleName ;
        String fnLastName   = lastName   ;
        String fnSuffix     = suffix     ;

        if ((title      != null && title.length      () > 0 ) &&
            ((firstName  != null && firstName.length  () > 0 ) ||
            (middleName != null && middleName.length () > 0 )  ||
            (lastName   != null && lastName.length   () > 0 )  ||
            (suffix     != null && suffix.length     () > 0 ))) {

            fnTitle = fnTitle + " ";

        }

        if ((firstName  != null && firstName.length  () > 0 ) &&
            ((middleName != null && middleName.length () > 0 ) ||
            (lastName   != null && lastName.length   () > 0 )  ||
            (suffix     != null && suffix.length     () > 0 ))) {

            fnFirstName = fnFirstName + " ";

        }

        if ((middleName != null && middleName.length () > 0 ) &&
            ((lastName   != null && lastName.length   () > 0 ) ||
            (suffix     != null && suffix.length     () > 0 ))) {

            fnMiddleName = fnMiddleName + " ";

        }

        if ((lastName != null && lastName.length () > 0) &&
            (suffix   != null && suffix.length   () > 0)) {

            fnLastName = fnLastName + " ";

        }

        value = "";

        if (fnTitle != null) {
            value = fnTitle;
        }

        if (fnFirstName != null) {
            value = value + fnFirstName;
        }

        if (fnMiddleName != null) {
            value = value + fnMiddleName;
        }

        if (fnLastName != null) {
            value = value + fnLastName;
        }

        if (fnSuffix != null) {
            value = value + fnSuffix;
        }

        if (value != null && value.length() > 0) {
            webDavContactMsg.append("<"           )
                            .append(WEBDAV_TAG_CN )
                            .append(">"           )
                            .append(value         )
                            .append("</"          )
                            .append(WEBDAV_TAG_CN )
                            .append(">\n"         );
        }

        value = (lastName != null ? lastName : "")                        +
                (((firstName != null && firstName.length() > 0)
                    && (lastName != null && lastName.length() > 0))
                    ? ", " : "")                                          +
                (firstName != null ? firstName : "")                      +
                (((middleName != null && middleName.length() > 0)
                    && (firstName !=null && firstName.length() > 0))
                    ? " "  : "")                                          +
                (middleName != null ? middleName : "")                    ;

        if (value != null) {
            webDavContactMsg.append("<"                )
                            .append(WEBDAV_TAG_FILE_AS )
                            .append(">"                )
                            .append(value              )
                            .append("</"               )
                            .append(WEBDAV_TAG_FILE_AS )
                            .append(">\n"              );
        }

        value = (lastName != null ? lastName : "")                        +
                (((firstName != null && firstName.length() > 0)
                    && (lastName != null && lastName.length() > 0))
                    ? " " : "")                                           +
                (firstName != null ? firstName : "")                      ;

        if (value != null) {
            webDavContactMsg.append("<"                 )
                            .append(WEBDAV_TAG_SUBJECT1 )
                            .append(">"                 )
                            .append(value               )
                            .append("</"                )
                            .append(WEBDAV_TAG_SUBJECT1 )
                            .append(">\n"               );
        }

        value = (lastName != null ? lastName : "")                        +
                (((firstName != null && firstName.length() > 0)
                    && (lastName != null && lastName.length() > 0 ))
                    ? " " : "")                                           +
                (firstName != null ? firstName : "")                      ;

        if (value != null) {
            webDavContactMsg.append("<"                 )
                            .append(WEBDAV_TAG_SUBJECT2 )
                            .append(">"                 )
                            .append(value               )
                            .append("</"                )
                            .append(WEBDAV_TAG_SUBJECT2 )
                            .append(">\n"               );
        }

        if (firstName != null) {
            webDavContactMsg.append("<"                   )
                            .append(WEBDAV_TAG_FIRST_NAME )
                            .append(">"                   )
                            .append(firstName             )
                            .append("</"                  )
                            .append(WEBDAV_TAG_FIRST_NAME )
                            .append(">\n"                 );
        }

        if (lastName != null) {
            webDavContactMsg.append("<"                  )
                            .append(WEBDAV_TAG_LAST_NAME )
                            .append(">"                  )
                            .append(lastName             )
                            .append("</"                 )
                            .append(WEBDAV_TAG_LAST_NAME )
                            .append(">\n"                );
        }

        if (middleName != null) {
            webDavContactMsg.append("<"                    )
                            .append(WEBDAV_TAG_MIDDLE_NAME )
                            .append(">"                    )
                            .append(middleName             )
                            .append("</"                   )
                            .append(WEBDAV_TAG_MIDDLE_NAME )
                            .append(">\n"                  );
        }

        if (language != null) {
            language  = StringTools.escapeXml(language);
            webDavContactMsg.append("<"                 )
                            .append(WEBDAV_TAG_LANGUAGE )
                            .append(">"                 )
                            .append(language            )
                            .append("</"                )
                            .append(WEBDAV_TAG_LANGUAGE )
                            .append(">\n"               );
        }

        if (suffix != null) {
            suffix  = StringTools.escapeXml(suffix);
            webDavContactMsg.append("<"               )
                            .append(WEBDAV_TAG_SUFFIX )
                            .append(">"               )
                            .append(suffix            )
                            .append("</"              )
                            .append(WEBDAV_TAG_SUFFIX )
                            .append(">\n"             );
        }

        if (title != null) {
            title  = StringTools.escapeXml(title);
            webDavContactMsg.append("<"              )
                            .append(WEBDAV_TAG_TITLE )
                            .append(">"              )
                            .append(title            )
                            .append("</"             )
                            .append(WEBDAV_TAG_TITLE )
                            .append(">\n"            );
        }

        if (initials != null) {
                    webDavContactMsg.append("<"                 )
                                    .append(WEBDAV_TAG_INITIALS )
                                    .append(">"                 )
                                    .append(initials            )
                                    .append("</"                )
                                    .append(WEBDAV_TAG_INITIALS )
                                    .append(">\n"               );
        }

        if (importance != null) {
                    webDavContactMsg.append("<"                        )
                                    .append(WEBDAV_TAG_IMPORTANCE      )
                                    .append(">"                        )
                                    .append(String.valueOf(importance) )
                                    .append("</"                       )
                                    .append(WEBDAV_TAG_IMPORTANCE      )
                                    .append(">\n"                      );
        }

        if (sensitivity != null) {
                    webDavContactMsg.append("<"                         )
                                    .append(WEBDAV_TAG_SENSITIVITY      )
                                    .append(">"                         )
                                    .append(String.valueOf(sensitivity) )
                                    .append("</"                        )
                                    .append(WEBDAV_TAG_SENSITIVITY      )
                                    .append(">\n"                       );
        }

        if (mileage != null) {
                    webDavContactMsg.append("<"                     )
                                    .append(WEBDAV_TAG_MILEAGE      )
                                    .append(">"                     )
                                    .append(mileage                 )
                                    .append("</"                    )
                                    .append(WEBDAV_TAG_MILEAGE      )
                                    .append(">\n"                   );
        }

        try {

            if (anniversary != null) {
                if (anniversary.length() > 0) {
                    webDavContactMsg.append("<")
                            .append(WEBDAV_TAG_ANNIVERSARY)
                            .append(">")
                            .append(DateTools.convertDayInWebDavDateFormat(
                                    anniversary, deviceTimeZone, true))
                            .append("</")
                            .append(WEBDAV_TAG_ANNIVERSARY)
                            .append(">\n");
                } else {
                    //todo find a way to send empty anniversary
                }
            }

            if (birthday != null) {
                if (birthday.length() > 0) {
                    webDavContactMsg.append("<")
                            .append(WEBDAV_TAG_BIRTHDAY)
                            .append(">")
                            .append(DateTools.convertDayInWebDavDateFormat(
                                    birthday, deviceTimeZone, true))
                            .append("</")
                            .append(WEBDAV_TAG_BIRTHDAY)
                            .append(">\n");
                } else {
                    //todo find a way to send empty birthday
                }
            }

        } catch (Exception e) {
            throw new DataAccessException(e.getMessage());
        }

        if (categories != null) {
    categories = StringTools.escapeXml(categories);
            webDavContactMsg.append("<")
                    .append(WEBDAV_TAG_CATEGORIES)
                    .append(">")
                    .append(categories.length() > 0 ? XmlParser. fieldToMultipleTag(categories) : "<x:v></x:v>")
                    .append("</")
                    .append(WEBDAV_TAG_CATEGORIES)
                    .append(">\n");
        }

        if (children != null) {
            children = StringTools.escapeXml(children);
            webDavContactMsg.append("<")
                    .append(WEBDAV_TAG_CHILDREN)
                    .append(">")
                    .append(children.length() > 0 ? XmlParser. fieldToMultipleTag(children) : "<x:v></x:v>")
                    .append("</")
                    .append(WEBDAV_TAG_CHILDREN)
                    .append(">\n");
        }

        if (hobby != null) {
            hobby  = StringTools.escapeXml(hobby);
            webDavContactMsg.append("<"                     )
                            .append(WEBDAV_TAG_HOBBY        )
                            .append(">"                     )
                            .append(hobby                   )
                            .append("</"                    )
                            .append(WEBDAV_TAG_HOBBY        )
                            .append(">\n"                   );
        }

        if (jobTitle != null) {
            jobTitle  = StringTools.escapeXml(jobTitle);
            webDavContactMsg.append("<"                  )
                            .append(WEBDAV_TAG_JOB_TITLE )
                            .append(">"                  )
                            .append(jobTitle             )
                            .append("</"                 )
                            .append(WEBDAV_TAG_JOB_TITLE )
                            .append(">\n"                );
        }

        if (officeLocation != null) {
            officeLocation  = StringTools.escapeXml(officeLocation);
            webDavContactMsg.append("<"                        )
                            .append(WEBDAV_TAG_OFFICE_LOCATION )
                            .append(">"                        )
                            .append(officeLocation             )
                            .append("</"                       )
                            .append(WEBDAV_TAG_OFFICE_LOCATION )
                            .append(">\n"                      );
        }

        if (organization != null) {
            organization  = StringTools.escapeXml(organization);
            webDavContactMsg.append("<"                )
                            .append(WEBDAV_TAG_COMPANY )
                            .append(">"                )
                            .append(organization       )
                            .append("</"               )
                            .append(WEBDAV_TAG_COMPANY )
                            .append(">\n"              );
        }

        if (homeStreet != null) {
            homeStreet  = StringTools.escapeXml(homeStreet);
            webDavContactMsg.append("<"                    )
                            .append(WEBDAV_TAG_HOME_STREET )
                            .append(">"                    )
                            .append(homeStreet             )
                            .append("</"                   )
                            .append(WEBDAV_TAG_HOME_STREET )
                            .append(">\n"                  );
        }

        if (homeCity != null) {
            homeCity  = StringTools.escapeXml(homeCity);
            webDavContactMsg.append("<"                  )
                            .append(WEBDAV_TAG_HOME_CITY )
                            .append(">"                  )
                            .append(homeCity             )
                            .append("</"                 )
                            .append(WEBDAV_TAG_HOME_CITY )
                            .append(">\n"                );
        }

        if (homeState != null) {
            homeState  = StringTools.escapeXml(homeState);
            webDavContactMsg.append("<"                   )
                            .append(WEBDAV_TAG_HOME_STATE )
                            .append(">"                   )
                            .append(homeState             )
                            .append("</"                  )
                            .append(WEBDAV_TAG_HOME_STATE )
                            .append(">\n"                 );
        }

        if (homePostalCode != null) {
            homePostalCode  = StringTools.escapeXml(homePostalCode);
            webDavContactMsg.append("<"                        )
                            .append(WEBDAV_TAG_HOME_POSTALCODE )
                            .append(">"                        )
                            .append(homePostalCode             )
                            .append("</"                       )
                            .append(WEBDAV_TAG_HOME_POSTALCODE )
                            .append(">\n"                      );
        }

        if (homePostOfficeBox != null) {
            homePostOfficeBox  = StringTools.escapeXml(homePostOfficeBox);
            webDavContactMsg.append("<"                            )
                            .append(WEBDAVTAG_HOME_POST_OFFICE_BOX )
                            .append(">"                            )
                            .append(homePostOfficeBox              )
                            .append("</"                           )
                            .append(WEBDAVTAG_HOME_POST_OFFICE_BOX )
                            .append(">\n"                          );
        }

        if (homeCountry != null) {
            homeCountry  = StringTools.escapeXml(homeCountry);
            webDavContactMsg.append("<"                     )
                            .append(WEBDAV_TAG_HOME_COUNTRY )
                            .append(">"                     )
                            .append(homeCountry             )
                            .append("</"                    )
                            .append(WEBDAV_TAG_HOME_COUNTRY )
                            .append(">\n"                   );
        }

        if (officeStreet != null) {
            officeStreet  = StringTools.escapeXml(officeStreet);
            webDavContactMsg.append("<"                    )
                            .append(WEBDAV_TAG_WORK_STREET )
                            .append(">"                    )
                            .append(officeStreet           )
                            .append("</"                   )
                            .append(WEBDAV_TAG_WORK_STREET )
                            .append(">\n"                  );
        }

        if (officeCity != null) {
            officeCity  = StringTools.escapeXml(officeCity);
            webDavContactMsg.append("<"                  )
                            .append(WEBDAV_TAG_WORK_CITY )
                            .append(">"                  )
                            .append(officeCity           )
                            .append("</"                 )
                            .append(WEBDAV_TAG_WORK_CITY )
                            .append(">\n"                );
        }

        if (officeState != null) {
            officeState  = StringTools.escapeXml(officeState);
            webDavContactMsg.append("<"                   )
                            .append(WEBDAV_TAG_WORK_STATE )
                            .append(">"                   )
                            .append(officeState           )
                            .append("</"                  )
                            .append(WEBDAV_TAG_WORK_STATE )
                            .append(">\n"                 );
        }

        if (officePostalCode != null) {
            officePostalCode  = StringTools.escapeXml(officePostalCode);
            webDavContactMsg.append("<"                        )
                            .append(WEBDAV_TAG_WORK_POSTALCODE )
                            .append(">"                        )
                            .append(officePostalCode           )
                            .append("</"                       )
                            .append(WEBDAV_TAG_WORK_POSTALCODE )
                            .append(">\n"                      );
        }

        if (officePostOfficeBox != null) {
            officePostOfficeBox  = StringTools.escapeXml(officePostOfficeBox);
            webDavContactMsg.append("<"                             )
                            .append(WEBDAV_TAG_WORK_POST_OFFICE_BOX )
                            .append(">"                             )
                            .append(officePostOfficeBox             )
                            .append("</"                            )
                            .append(WEBDAV_TAG_WORK_POST_OFFICE_BOX )
                            .append(">\n"                           );
        }

        if (officeCountry != null) {
            officeCountry  = StringTools.escapeXml(officeCountry);
            webDavContactMsg.append("<"                     )
                            .append(WEBDAV_TAG_WORK_COUNTRY )
                            .append(">"                     )
                            .append(officeCountry           )
                            .append("</"                    )
                            .append(WEBDAV_TAG_WORK_COUNTRY )
                            .append(">\n"                   );
        }

        if (otherStreet != null) {
            otherStreet  = StringTools.escapeXml(otherStreet);
            webDavContactMsg.append("<"                     )
                            .append(WEBDAV_TAG_OTHER_STREET )
                            .append(">"                     )
                            .append(otherStreet             )
                            .append("</"                    )
                            .append(WEBDAV_TAG_OTHER_STREET )
                            .append(">\n"                   );
        }

        if (otherCity != null) {
            otherCity  = StringTools.escapeXml(otherCity);
            webDavContactMsg.append("<"                   )
                            .append(WEBDAV_TAG_OTHER_CITY )
                            .append(">"                   )
                            .append(otherCity             )
                            .append("</"                  )
                            .append(WEBDAV_TAG_OTHER_CITY )
                            .append(">\n"                 );
        }

        if (otherState != null) {
            otherState  = StringTools.escapeXml(otherState);
            webDavContactMsg.append("<"                    )
                            .append(WEBDAV_TAG_OTHER_STATE )
                            .append(">"                    )
                            .append(otherState             )
                            .append("</"                   )
                            .append(WEBDAV_TAG_OTHER_STATE )
                            .append(">\n"                  );
        }

        if (otherPostalCode != null) {
            otherPostalCode  = StringTools.escapeXml(otherPostalCode);
            webDavContactMsg.append("<"                         )
                            .append(WEBDAV_TAG_OTHER_POSTALCODE )
                            .append(">"                         )
                            .append(otherPostalCode             )
                            .append("</"                        )
                            .append(WEBDAV_TAG_OTHER_POSTALCODE )
                            .append(">\n"                       );
        }

        if (otherPostOfficeBox != null) {
            otherPostOfficeBox  = StringTools.escapeXml(otherPostOfficeBox);
            webDavContactMsg.append("<"                              )
                            .append(WEBDAV_TAG_OTHER_POST_OFFICE_BOX )
                            .append(">"                              )
                            .append(otherPostOfficeBox               )
                            .append("</"                             )
                            .append(WEBDAV_TAG_OTHER_POST_OFFICE_BOX )
                            .append(">\n"                            );
        }


        if (otherCountry != null) {
            otherCountry  = StringTools.escapeXml(otherCountry);
            webDavContactMsg.append("<"                      )
                            .append(WEBDAV_TAG_OTHER_COUNTRY )
                            .append(">"                      )
                            .append(otherCountry             )
                            .append("</"                     )
                            .append(WEBDAV_TAG_OTHER_COUNTRY )
                            .append(">\n"                    );
        }

        if (officePhone != null) {
            officePhone  = StringTools.escapeXml(officePhone);
            webDavContactMsg.append("<"                   )
                            .append(WEBDAV_TAG_WORK_PHONE )
                            .append(">"                   )
                            .append(officePhone           )
                            .append("</"                  )
                            .append(WEBDAV_TAG_WORK_PHONE )
                            .append(">\n"                 );
        }

        if (homePhone != null) {
            homePhone  = StringTools.escapeXml(homePhone);
            webDavContactMsg.append("<"                   )
                            .append(WEBDAV_TAG_HOME_PHONE )
                            .append(">"                   )
                            .append(homePhone             )
                            .append("</"                  )
                            .append(WEBDAV_TAG_HOME_PHONE )
                            .append(">\n"                 );
        }

        if (mobilePhone != null) {
            mobilePhone  = StringTools.escapeXml(mobilePhone);
            webDavContactMsg.append("<"                     )
                            .append(WEBDAV_TAG_MOBILE_PHONE )
                            .append(">"                     )
                            .append(mobilePhone             )
                            .append("</"                    )
                            .append(WEBDAV_TAG_MOBILE_PHONE )
                            .append(">\n"                   );
        }

        if (assistantPhone != null) {
            assistantPhone  = StringTools.escapeXml(assistantPhone);
            webDavContactMsg.append("<"                        )
                            .append(WEBDAV_TAG_ASSISTANT_PHONE )
                            .append(">"                        )
                            .append(assistantPhone             )
                            .append("</"                       )
                            .append(WEBDAV_TAG_ASSISTANT_PHONE )
                            .append(">\n"                      );
        }

        if (officeOtherPhone != null) {
            officeOtherPhone  = StringTools.escapeXml(officeOtherPhone);
            webDavContactMsg.append("<"                         )
                            .append(WEBDAV_TAG_WORK_OTHER_PHONE )
                            .append(">"                         )
                            .append(officeOtherPhone            )
                            .append("</"                        )
                            .append(WEBDAV_TAG_WORK_OTHER_PHONE )
                            .append(">\n"                       );
        }

        if (officeFax != null) {
            officeFax  = StringTools.escapeXml(officeFax);
            webDavContactMsg.append("<"                 )
                            .append(WEBDAV_TAG_WORK_FAX )
                            .append(">"                 )
                            .append(officeFax           )
                            .append("</"                )
                            .append(WEBDAV_TAG_WORK_FAX )
                            .append(">\n"               );
        }

        if (callBackPhone != null) {
            callBackPhone  = StringTools.escapeXml(callBackPhone);
            webDavContactMsg.append("<"                       )
                            .append(WEBDAV_TAG_CALLBACK_PHONE )
                            .append(">"                       )
                            .append(callBackPhone             )
                            .append("</"                      )
                            .append(WEBDAV_TAG_CALLBACK_PHONE )
                            .append(">\n"                     );
        }

        if (mobileOtherPhone != null) {
            mobileOtherPhone  = StringTools.escapeXml(mobileOtherPhone);
            webDavContactMsg.append("<"                           )
                            .append(WEBDAV_TAG_MOBILE_OTHER_PHONE )
                            .append(">"                           )
                            .append(mobileOtherPhone              )
                            .append("</"                          )
                            .append(WEBDAV_TAG_MOBILE_OTHER_PHONE )
                            .append(">\n"                         );
        }


        if (organizationPhone != null) {
            organizationPhone  = StringTools.escapeXml(organizationPhone);
            webDavContactMsg.append("<"                      )
                            .append(WEBDAV_TAG_COMPANY_PHONE )
                            .append(">"                      )
                            .append(organizationPhone        )
                            .append("</"                     )
                            .append(WEBDAV_TAG_COMPANY_PHONE )
                            .append(">\n"                    );
        }

        if (homeOtherPhone != null) {
            homeOtherPhone  = StringTools.escapeXml(homeOtherPhone);
            webDavContactMsg.append("<"                         )
                            .append(WEBDAV_TAG_HOME_OTHER_PHONE )
                            .append(">"                         )
                            .append(homeOtherPhone              )
                            .append("</"                        )
                            .append(WEBDAV_TAG_HOME_OTHER_PHONE )
                            .append(">\n"                       );
        }

        if (homeFax != null) {
            homeFax  = StringTools.escapeXml(homeFax);
            webDavContactMsg.append("<"                 )
                            .append(WEBDAV_TAG_HOME_FAX )
                            .append(">"                 )
                            .append(homeFax             )
                            .append("</"                )
                            .append(WEBDAV_TAG_HOME_FAX )
                            .append(">\n"               );
        }

        if (otherPhone != null) {
            otherPhone  = StringTools.escapeXml(otherPhone);
            webDavContactMsg.append("<"                    )
                            .append(WEBDAV_TAG_OTHER_PHONE )
                            .append(">"                    )
                            .append(otherPhone             )
                            .append("</"                   )
                            .append(WEBDAV_TAG_OTHER_PHONE )
                            .append(">\n"                  );
        }

        if (otherFax != null) {
            otherFax  = StringTools.escapeXml(otherFax);
            webDavContactMsg.append("<"                  )
                            .append(WEBDAV_TAG_OTHER_FAX )
                            .append(">"                  )
                            .append(otherFax             )
                            .append("</"                 )
                            .append(WEBDAV_TAG_OTHER_FAX )
                            .append(">\n"                );
        }

        if (pager != null) {
            pager  = StringTools.escapeXml(pager);
            webDavContactMsg.append("<"              )
                            .append(WEBDAV_TAG_PAGER )
                            .append(">"              )
                            .append(pager            )
                            .append("</"             )
                            .append(WEBDAV_TAG_PAGER )
                            .append(">\n"            );
        }

        if (radioPhone != null )
        {
            radioPhone = StringTools.escapeXml(radioPhone);
            webDavContactMsg.append("<"              )
                            .append(WEBDAV_TAG_RADIO_PHONE)
                            .append(">"              )
                            .append(radioPhone       )
                            .append("</"             )
                            .append(WEBDAV_TAG_RADIO_PHONE)
                            .append(">\n"            );
        }

        if (department != null) {
            department  = StringTools.escapeXml(department);
            webDavContactMsg.append("<"                   )
                            .append(WEBDAV_TAG_DEPARTMENT )
                            .append(">"                   )
                            .append(department            )
                            .append("</"                  )
                            .append(WEBDAV_TAG_DEPARTMENT )
                            .append(">\n"                 );
        }

        if (profession != null) {
            profession  = StringTools.escapeXml(profession);
            webDavContactMsg.append("<"                   )
                            .append(WEBDAV_TAG_PROFESSION )
                            .append(">"                   )
                            .append(profession            )
                            .append("</"                  )
                            .append(WEBDAV_TAG_PROFESSION )
                            .append(">\n"                 );
        }

        if (nickName != null) {
            nickName  = StringTools.escapeXml(nickName);
            webDavContactMsg.append("<"                 )
                            .append(WEBDAV_TAG_NICKNAME )
                            .append(">"                 )
                            .append(nickName            )
                            .append("</"                )
                            .append(WEBDAV_TAG_NICKNAME )
                            .append(">\n"               );
        }

        if (spouse != null) {
            spouse  = StringTools.escapeXml(spouse);
            webDavContactMsg.append("<"               )
                            .append(WEBDAV_TAG_SPOUSE )
                            .append(">"               )
                            .append(spouse            )
                            .append("</"              )
                            .append(WEBDAV_TAG_SPOUSE )
                            .append(">\n"             );
        }

        if (manager != null) {
            manager  = StringTools.escapeXml(manager);
            webDavContactMsg.append("<"                )
                            .append(WEBDAV_TAG_MANAGER )
                            .append(">"                )
                            .append(manager            )
                            .append("</"               )
                            .append(WEBDAV_TAG_MANAGER )
                            .append(">\n"              );
        }

        if (assistant != null) {
            assistant  = StringTools.escapeXml(assistant);
            webDavContactMsg.append("<"                  )
                            .append(WEBDAV_TAG_ASSISTANT )
                            .append(">"                  )
                            .append(assistant            )
                            .append("</"                 )
                            .append(WEBDAV_TAG_ASSISTANT )
                            .append(">\n"                );
        }

        if (personalHomePage != null) {
            personalHomePage  = StringTools.escapeXml(personalHomePage);
            webDavContactMsg.append("<"                           )
                            .append(WEBDAV_TAG_PERSONAL_HOME_PAGE )
                            .append(">"                           )
                            .append(personalHomePage              )
                            .append("</"                          )
                            .append(WEBDAV_TAG_PERSONAL_HOME_PAGE )
                            .append(">\n"                         );
        }

        if (businessHomePage != null) {
            businessHomePage  = StringTools.escapeXml(businessHomePage);
            webDavContactMsg.append("<"                           )
                            .append(WEBDAV_TAG_BUSINESS_HOME_PAGE )
                            .append(">"                           )
                            .append(businessHomePage              )
                            .append("</"                          )
                            .append(WEBDAV_TAG_BUSINESS_HOME_PAGE )
                            .append(">\n"                         );
        }

        if (emailAddress1 != null) {
            emailAddress1  = StringTools.escapeXml(emailAddress1);
            webDavContactMsg.append("<"               )
                            .append(WEBDAV_TAG_EMAIL1 )
                            .append(">"               )
                            .append(emailAddress1     )
                            .append("</"              )
                            .append(WEBDAV_TAG_EMAIL1 )
                            .append(">\n"             );

        }

        if (emailAddress2 != null) {
            emailAddress2  = StringTools.escapeXml(emailAddress2);
            webDavContactMsg.append("<"               )
                            .append(WEBDAV_TAG_EMAIL2 )
                            .append(">"               )
                            .append(emailAddress2     )
                            .append("</"              )
                            .append(WEBDAV_TAG_EMAIL2 )
                            .append(">\n"             );
        }

        if (emailAddress3 != null) {
            emailAddress3  = StringTools.escapeXml(emailAddress3);
            webDavContactMsg.append("<"               )
                            .append(WEBDAV_TAG_EMAIL3 )
                            .append(">"               )
                            .append(emailAddress3     )
                            .append("</"              )
                            .append(WEBDAV_TAG_EMAIL3 )
                            .append(">\n"             );
        }

        if (notes!=null)
        {
            webDavContactMsg.append("<").append(WEBDAV_TAG_NOTES).append(">").
                             append(notes).
                             append("</").append(WEBDAV_TAG_NOTES).append(">\n");
        }

        webDavContactMsg.append("</D:prop>\n"         )
                        .append("</D:set>\n"          );
        if ("".equals(birthday) || "".equals(anniversary)) {
            webDavContactMsg.append("<D:remove>\n<D:prop>\n");
            if ("".equals(birthday)) {
                webDavContactMsg.append("<").append(WEBDAV_TAG_BIRTHDAY).append(">").
                        append("</").append(WEBDAV_TAG_BIRTHDAY).append(">\n");
            }
            if ("".equals(anniversary)) {
                webDavContactMsg.append("<").append(WEBDAV_TAG_ANNIVERSARY).append(">").
                        append("</").append(WEBDAV_TAG_ANNIVERSARY).append(">\n");
            }
            webDavContactMsg.append("</D:prop>\n</D:remove>\n");
        }
        webDavContactMsg.append("</D:propertyupdate>" );

        webDavHeaderMsg
            = MessageFormat.format(ItemDAO.WEBDAV_HEADER_PROPPATCH,
                new Object[] {
                        "/"                   +
                        server                +
                        "/"                   +
                        username              +
                        "/"                   +
                        resource              ,
                        c.getHref()
              }
        );


        try {
             response = this.webDavHttp.sendRequest(webDavHeaderMsg             ,
                                                    credentials                 ,
                                                    webDavContactMsg.toString() ,
                                                    FILE_ENCODING               );
        } catch (Exception e) {
             throw new DataAccessException("Error getting Exchange server response",
                                            e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                       +
                                          username                      +
                                          " URI "                       +
                                          exchangeFolder                +
                                          " setting exchange contact"   ,
                                          e)                            ;

        }

        try {
            id = XmlParser.getRuidFromResponse(response);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error getting resource id", e);
        }

        c.setId(id);

        return c;

    }

    /**
     * Delete a <i>Contact</i> from Exchange server.
     *
     * @param contact
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @throws com.funambol.exchange.DataAccessException
     *
     */
    public void removeContact(Contact contact     ,
                              String  username    ,
                              String  credentials ,
                              String  exchangeFolder)
    throws DataAccessException {

        String webDavContactMsg        ;

        String webDavHeaderMsg         ;
        String response                ;

        String server                  ;
        String resource                ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        webDavContactMsg       = ""                      ;

        webDavHeaderMsg =
            MessageFormat.format(ItemDAO.WEBDAV_HEADER_REMOVE,
            new Object[] {
                        "/"                    +
                            server             +
                            "/"                +
                            username           +
                            "/"                +
                            resource           ,
                            contact.getHref()
            }
        );

        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials, webDavContactMsg, FILE_ENCODING);
        } catch (Exception e) {
            throw new DataAccessException("Error getting Exchange server response",
                                          e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                      +
                                          username                     +
                                          " URI "                      +
                                          exchangeFolder               +
                                          " removing exchange contact" ,
                                          e)                           ;

        }
    }

    /**
     * get contacts from Exchange Server
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     * @return array of find contacts
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Contact[] getContacts (String    username    ,
                                  String    credentials ,
                                  String[]  ids         ,
                                  String    exchangeFolder)
        throws DataAccessException {

        String clause;

        clause = getClause(ids);

        return getContacts(username, credentials, clause, exchangeFolder);

    }

    /**
     * get Contact array with setting fields only for twins
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     * @param ids
     *
     * @return Contact array with setting fields only for twins
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Contact[] getContactsForTwin(String   username       ,
                                        String   credentials    ,
                                        String[] ids            ,
                                        String   exchangeFolder )
    throws DataAccessException {

        ArrayList contacts             ;
        Contact   c                    ;

        String[]  props                ;
        String    prop                 ;
        String[]  resp                 ;

        String    replUid              ;
        String    uri                  ;
        //String    creationDate         = null ;
        String    lastUpdate           ;

        String    firstName            ;
        String    lastName             ;

        String    isFolder             ;

        String    response             ;

        String    webDavBodyMsg        ;

        String    webDavHeaderMsg      ;

        String    server               ;
        String    resource             ;

        int       count                = 0    ;

        String    clause               ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        clause = getClause(ids);

        webDavBodyMsg =
                    MessageFormat.format(WEBDAV_MSG_SELECT_CONTACTS_FOR_TWIN,
            new Object[] {server, username, resource, clause});

        webDavHeaderMsg =
            MessageFormat.format(WEBDAV_HEADER_SELECT,

            new Object[] {
                    "/"                  +
                        server           +
                        "/"              +
                        username         +
                        "/"              +
                        resource
            }
        );


        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg ,
                                                   credentials     ,
                                                   webDavBodyMsg   ,
                                                   FILE_ENCODING   );
        } catch (Exception e) {
            throw new DataAccessException("Error getting Exchange server response",
                                           e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                    +
                                          username                   +
                                          " URI "                    +
                                          exchangeFolder             +
                                          " reading exchange items"  ,
                                          e)                         ;
        }

        resp = new String[] {response};

        try {
            props = XmlParser.getXMLTag(resp, TAG_RESPONSE);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing item", e);
        }

        contacts = new ArrayList();

        for (int i=0, l = props.length; i < l; i++) {

            try {

                prop = props[i];

                isFolder      = XmlParser.getXMLInitTagValue
                                 (prop, TAG_IS_FOLDER)        ;

                replUid       = XmlParser.getXMLTagValue
                                 (prop, TAG_REPLUID)          ;

                uri           = XmlParser.getXMLTagValue
                                 (prop, TAG_HREF)             ;

                //creationDate  = XmlParser.getXMLInitTagValue
                //                 (prop, TAG_CREATION_DATE)    ;

                lastUpdate    = XmlParser.getXMLInitTagValue
                                 (prop, TAG_LAST_MODIFIED)    ;

                firstName = XmlParser.getXMLInitTagValue (
                                prop, TAG_FIRSTNAME);

                lastName  = XmlParser.getXMLInitTagValue (
                                prop, TAG_LASTNAME);

                c = new Contact(getIdFromReplUid(replUid), DateTools.webDavTagToDate(lastUpdate));

                c.setHref( XmlParser.getHrefFromUri(uri));

                if (firstName != null) {
                    c.getName().getFirstName().setPropertyValue(firstName);
                }

                if (lastName != null) {
                    c.getName().getLastName().setPropertyValue(lastName);
                }

                if(PROP_NO_FOLDER.equals(isFolder)) {
                    contacts.add(c);
                    count ++;
                }

            } catch (XmlParseException e) {
                throw new DataAccessException("Error parsing item", e);
            }

        }

        return (Contact[]) contacts.toArray(new Contact[count]);
    }

    //------------------------------------------------------------- Private methods

    /**
     * Create a <i>Contact</i> array object from WebDav msg.
     *
     * @param webDavMsg
     *
     * @return a newly create <i>Calendar</i> object initialized with the
     *         fields in the result set
     *
     */
    private Contact[] getContactsFromWebDavMsg(String webDavMsg)
    throws DataAccessException {

        ArrayList  contacts   ;
        String []  resps      ;
        String []  msg        ;

        String     response   ;
        String     id         ;
        String     replUid    ;

        String     isFolder   ;

        int        count      = 0    ;

        msg = new String[] {webDavMsg};

        contacts = new ArrayList();

        try {

            resps = XmlParser.getXMLTag(msg, TAG_RESPONSE);

            for (int i = 0, l = resps.length; i < l; i++) {

                response    =  resps[i]                                        ;
                replUid     =  XmlParser.getXMLInitTagValue(response,
                                                        TAG_REPLUID)           ;

                id          =  getIdFromReplUid(replUid)                       ;

                isFolder    =  XmlParser.getXMLInitTagValue(response,
                                                            TAG_IS_FOLDER)     ;

                if(PROP_NO_FOLDER.equals(isFolder)) {
                    contacts.add(getContactFromResponseTag(id, response));
                    count ++;
                }

            }

        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing contact", e);
        }

        return (Contact[])contacts.toArray(new Contact[count]);

    }

    /**
     * Create a <i>Contact</i> object from response tag.
     *
     * @param id
     * @param msgResponse
     *
     * @return a newly create <i>Contact</i> object initialized with the
     *         fields in the result set
     *
     */
    private Contact getContactFromResponseTag(String id, String msgResponse)
    throws DataAccessException {

        Contact c           ;
        String  lastUpdate  ;

        String  value       ;

        Title             title                  ;
        Phone             phone                  ;
        Email             email                  ;
        WebPage           webPage                ;

        String            uri                    ;

        String            anniversary            ;
        String            birthday               ;
        String            categories             ;
        String            children               ;
        String            displayName            ;
        String            firstName              ;
        String            hobby                  ;
        String            importance             ;
        String            initials               ;
        String            lastName               ;
        String            language               ;
        String            middleName             ;
        String            mileage                ;
        String            nickName               ;
        String            sensitivity            ;
        String            spouse                 ;
        String            suffix                 ;
        String            salutation             ;

        String            homeStreet             ;
        String            homeCity               ;
        String            homeState              ;
        String            homePostOfficeBox      ;
        String            homePostalCode         ;
        String            homeCountry            ;

        String            officeStreet           ;
        String            officeCity             ;
        String            officeState            ;
        String            officeCountry          ;
        String            officePostalCode       ;
        String            officePostOfficeBox    ;

        String            otherStreet            ;
        String            otherCity              ;
        String            otherState             ;
        String            otherPostalCode        ;
        String            otherCountry           ;

        String            assistantPhone         ;
        String            callBackPhone          ;
        String            homeFax                ;
        String            homePhone              ;
        String            homeOtherPhone         ;
        String            mobileOtherPhone       ;
        String            mobilePhone            ;
        String            officeFax              ;
        String            officeOtherPhone       ;
        String            officePhone            ;
        String            organizationPhone      ;
        String            otherFax               ;
        String            otherPhone             ;
        String            pager                  ;
        String radioPhone;

        String            assistant              ;
        String            department             ;
        String            jobTitle               ;
        String            manager                ;
        String            officeLocation         ;
        String            organization           ;
        String            profession             ;

        String            emailAddress1          = null ;
        String            emailAddress2          = null ;
        String            emailAddress3          = null ;
       String            businessHomePage       ;
        String            personalHomePage       ;
        String notes;

        try {

            uri         = XmlParser.getXMLTagValue
                             (msgResponse, TAG_HREF);

            anniversary = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_ANNIVERSARY
                          );

            birthday  = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_BIRTHDAY
                        );

            categories
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_CATEGORIES
                          );

            children
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_CHILDREN
                          );

            displayName = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_FILEAS
                          );

            firstName = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_FIRSTNAME
                        );
            hobby = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_HOBBY
                    );

            importance = XmlParser.getXMLInitTagValue (
                          msgResponse,
                          TAG_IMPORTANCE
                         );

            initials = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_INITIALS
                       );

            language = XmlParser.getXMLInitTagValue (
                           msgResponse,
                           TAG_LANGUAGE
                       );

            lastName  = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_LASTNAME
                        );

            middleName = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_MIDDLENAME
                         );

            mileage
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_MILEAGE
            );

            nickName = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_NICKNAME
                       );

            sensitivity
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_SENSITIVITY
                        );

            spouse = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_SPOUSECN
                     );

            suffix = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_SUFFIX
            );

            salutation = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_TITLE
            );


            homeStreet =  XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_STREET
                          );

            homeCity = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_CITY
                       );

            homeState = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_STATE
                        );

            homePostalCode = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_HOME_POSTAL_CODE
                             );

            homePostOfficeBox
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_POST_OFFICE_BOX
            );

            homeCountry = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_COUNTRY
                          );


            officeStreet = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_WORK_STREET
                           );

            officeCity = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_WORK_CITY
                         );

            officeState = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_WORK_STATE
                          );

            officePostalCode = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_WORK_POSTALCODE
                               );

            officePostOfficeBox
                            = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_WORK_POST_OFFICE_BOX
                              );

            officeCountry = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_WORK_COUNTRY
                            );


            otherStreet = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_STREET
                          );

            otherCity = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_CITY
                        );

            otherState = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_STATE
                         );

            otherPostalCode = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_OTHER_POSTALCODE
                              );

            otherCountry = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_COUNTRY
                           );


            assistant = XmlParser.getXMLInitTagValue (
                              msgResponse,
                              TAG_SECRETARY
                          );

            department = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_DEPARTMENT
                         );

            jobTitle = XmlParser.getXMLInitTagValue (
                           msgResponse,
                           TAG_JOBTITLE
                       );

            manager = XmlParser.getXMLInitTagValue (
                         msgResponse,
                         TAG_MANAGER
                       );

            officeLocation
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OFFICE_LOCATION
            );

            organization =  XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_ORGANIZATION
                            );

            profession = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_PROFESSION
                         );


            assistantPhone = XmlParser.getXMLInitTagValue (
                                 msgResponse,
                                 TAG_SECRETARY_PHONE
                              );


            callBackPhone = XmlParser.getXMLInitTagValue (
                                 msgResponse,
                                 TAG_CALL_BACK_PHONE
                             );


            homeFax = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_HOME_FAX
                      );

            homeOtherPhone = XmlParser.getXMLInitTagValue (
                                 msgResponse,
                                 TAG_HOME_PHONE2
                             );


            homePhone = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_PHONE
                         );

            mobileOtherPhone = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_OTHER_MOBILE
                               );


            mobilePhone = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_MOBILE
                          );

            officeFax = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_FAC_SIMILE_TELEPHONE_NUMBER
                        );


            officePhone = XmlParser.getXMLInitTagValue (
                             msgResponse,
                             TAG_TELEPHONE_NUMBER
                           );

            officeOtherPhone = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_TELEPHONE_NUMBER2
                               );

            organizationPhone = XmlParser.getXMLInitTagValue (
                                    msgResponse,
                                    TAG_ORGANIZATION_MAIN_PHONE
                                );


            otherFax = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_OTHER_FAX
                       );

            otherPhone = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_TELEPHONE
                         );


            pager = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_PAGER
                    );

            radioPhone = XmlParser.getXMLInitTagValue(msgResponse,
                    TAG_RADIO_PHONE);

            value =
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL1
                );

            if (value != null && value.trim().length() > 0) {

                value = value.replaceAll("&lt;", "<");
                value = value.replaceAll("&gt;", ">");

                if (value.indexOf("<") != -1 && value.indexOf(">") != -1) {

                    emailAddress1 = value.substring(value.indexOf("<") + 1,
                                                    value.indexOf(">"));
                } else {
                    emailAddress1 = value;
                }

            }

            emailAddress1 = StringTools.normalizeEmailAddress(emailAddress1);

            value =
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL2
                );

            if (value != null && value.trim().length() > 0) {

                value = value.replaceAll("&lt;", "<");
                value = value.replaceAll("&gt;", ">");

                if (value.indexOf("<") != -1 && value.indexOf(">") != -1) {

                    emailAddress2 = value.substring(value.indexOf("<") + 1,
                                                        value.indexOf(">"));
                } else {
                    emailAddress2 = value;

                }
            }

            emailAddress2 = StringTools.normalizeEmailAddress(emailAddress2);

            value =
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL3
                );

            if (value != null && value.trim().length() > 0) {

                value = value.replaceAll("&lt;", "<");
                value = value.replaceAll("&gt;", ">");

                if (value.indexOf("<") != -1 && value.indexOf(">") != -1) {

                    emailAddress3 = value.substring(value.indexOf("<") + 1,
                                                        value.indexOf(">"));
                } else {
                    emailAddress3 = value;

                }
            }

            emailAddress3 = StringTools.normalizeEmailAddress(emailAddress3);

            businessHomePage = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_BUSINESS_HOME_PAGE
                             );

            personalHomePage
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_PERSONAL_HOME_PAGE
            );

            notes = XmlParser.getXMLInitTagValue(msgResponse, TAG_NOTES);

            lastUpdate =
                XmlParser.getXMLInitTagValue(msgResponse,
                TAG_LAST_MODIFIED);



            c = new Contact(id, DateTools.webDavTagToDate(lastUpdate));

            c.setHref( XmlParser.getHrefFromUri(uri));

            if (firstName != null) {
                c.getName().getFirstName().setPropertyValue(firstName);
            }

            if (lastName != null) {
                c.getName().getLastName().setPropertyValue(lastName);
            }

            if (middleName != null) {
                c.getName().getMiddleName().setPropertyValue(middleName);
            }

            if (nickName != null) {
                c.getName().getNickname().setPropertyValue(nickName);
            }

            if (displayName != null) {
                c.getName().getDisplayName().setPropertyValue(displayName);
            }

            if (suffix != null) {
                c.getName().getSuffix().setPropertyValue(suffix);
            }

            if (importance != null) {
                c.setImportance(new Short(importance));
            }

            if (initials != null) {
                c.getName().getInitials().setPropertyValue(initials);
            }

            //workaround needed in order for the delete from server to client of the categories field to work
            if (categories == null || categories.length() < 1) {
                categories = "<c:v></c:v>";
            }
            c.getCategories().setPropertyValue(XmlParser.getFieldFromMultipleTag(categories));

            if (salutation != null) {
                c.getName().getSalutation().setPropertyValue(salutation);
            }

            if (mileage != null) {
                c.setMileage(mileage);
            }

            if (sensitivity != null) {
                c.setSensitivity(new Short(sensitivity));
            }

            if (jobTitle != null) {
                title = new Title();
                title.setTitleType(FIELD_TYPE_JOB_TITLE);
                title.setPropertyValue(jobTitle);
                c.getBusinessDetail().addTitle(title);
            }

            if (organization != null) {
                c.getBusinessDetail().getCompany().setPropertyValue(organization);
            }

            if (department != null) {
                c.getBusinessDetail().getDepartment().setPropertyValue(department);
            }

            if (profession != null) {
                c.getBusinessDetail().getRole().setPropertyValue(profession);
            }

            if (anniversary != null) {
                c.getPersonalDetail().
                    setAnniversary(DateTools.
                        convertWebDavDateInDayFormat(anniversary, deviceTimeZone, true));
            }

            if (birthday != null) {
                c.getPersonalDetail().
                    setBirthday(DateTools.
                        convertWebDavDateInDayFormat(birthday, deviceTimeZone, true));
            }

            //workaround needed in order for the delete from server to client of the children field to work
            if (children == null || children.length() < 1) {
                children = "<c:v></c:v>";
            }
            c.getPersonalDetail().setChildren(XmlParser.getFieldFromMultipleTag(children));

            if (hobby != null) {
                c.getPersonalDetail().setHobbies(hobby);
            }

            if (language != null) {
                c.setLanguages(language);
            }

            if (spouse != null) {
                c.getPersonalDetail().setSpouse(spouse);
            }

            if (manager != null) {
                c.getBusinessDetail().setManager(manager);
            }

            if (assistant != null) {
                c.getBusinessDetail().setAssistant(assistant);
            }

            if (officeLocation != null) {
                c.getBusinessDetail().setOfficeLocation(officeLocation);
            }

            if (officePhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_BUSINESS_TELEPHONE_NUMBER);
                phone.setPropertyValue(officePhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (homePhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_HOME_TELEPHONE_NUMBER);
                phone.setPropertyValue(homePhone);
                c.getPersonalDetail().addPhone(phone);
            }

            if (mobilePhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_MOBILE_TELEPHONE_NUMBER);
                phone.setPropertyValue(mobilePhone);
                c.getPersonalDetail().addPhone(phone);
            }

            if (assistantPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_ASSISTANT_NUMBER);
                phone.setPropertyValue(assistantPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (officeOtherPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_BUSINESS_OTHER_TELEPHONE_NUMBER);
                phone.setPropertyValue(officeOtherPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (officeFax != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_BUSINESS_FAX_NUMBER);
                phone.setPropertyValue(officeFax);
                c.getBusinessDetail().addPhone(phone);
            }

            if (callBackPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_CALLBACK_NUMBER);
                phone.setPropertyValue(callBackPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (mobileOtherPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_CAR_TELEPHONE_NUMBER);
                phone.setPropertyValue(mobileOtherPhone);
                c.getPersonalDetail().addPhone(phone);
            }

            if (organizationPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_COMPANY_MAIN_TELEPHONE_NUMBER);
                phone.setPropertyValue(organizationPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (homeOtherPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_HOME_OTHER_TELEPHONE_NUMBER);
                phone.setPropertyValue(homeOtherPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (homeFax != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_HOME_FAX_NUMBER);
                phone.setPropertyValue(homeFax);
                c.getBusinessDetail().addPhone(phone);
            }

            if (otherPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_OTHER_TELEPHONE_NUMBER);
                phone.setPropertyValue(otherPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (otherFax != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_OTHER_FAX_NUMBER);
                phone.setPropertyValue(otherFax);
                c.getBusinessDetail().addPhone(phone);
            }

            if (pager != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_PAGER_NUMBER);
                phone.setPropertyValue(pager);
                c.getBusinessDetail().addPhone(phone);
            }

            if (radioPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_RADIO_PHONE);
                phone.setPropertyValue(radioPhone);
                c.getPersonalDetail().addPhone(phone);
            }

            if (businessHomePage != null) {
                webPage = new WebPage();
                webPage.setWebPageType(FIELD_TYPE_BUSINESS_HOME_PAGE);
                webPage.setPropertyValue(businessHomePage);
                c.getBusinessDetail().addWebPage(webPage);
            }

            if (personalHomePage != null) {
                webPage = new WebPage();
                webPage.setWebPageType(FIELD_TYPE_PERSONAL_HOME_PAGE);
                webPage.setPropertyValue(personalHomePage);
                c.getPersonalDetail().addWebPage(webPage);
            }

            if (emailAddress1 != null) {
                email = new Email();
                email.setEmailType(FIELD_TYPE_EMAIL1_ADDRESS);
                email.setPropertyValue(emailAddress1);
                c.getPersonalDetail().addEmail(email);
            }

            if (emailAddress2 != null) {
                email = new Email();
                email.setEmailType(FIELD_TYPE_EMAIL2_ADDRESS);
                email.setPropertyValue(emailAddress2);
                c.getPersonalDetail().addEmail(email);
            }

            if (emailAddress3 != null) {
                email = new Email();
                email.setEmailType(FIELD_TYPE_EMAIL3_ADDRESS);
                email.setPropertyValue(emailAddress3);
                c.getBusinessDetail().addEmail(email);
            }

            if (homeStreet != null) {
                c.getPersonalDetail().getAddress().
                                        getStreet().
                                            setPropertyValue(homeStreet);
            }

            if (homeCity != null) {
                c.getPersonalDetail().getAddress().
                                        getCity().
                                            setPropertyValue(homeCity);
            }

            if (homeState != null) {
                c.getPersonalDetail().getAddress().
                                        getState().
                                            setPropertyValue(homeState);
            }

            if (homePostalCode != null) {
                c.getPersonalDetail().getAddress().
                                        getPostalCode().
                                            setPropertyValue(homePostalCode);
            }

            if (homePostOfficeBox != null) {
                c.getPersonalDetail().getAddress().
                                        getPostOfficeAddress().
                                            setPropertyValue(homePostOfficeBox);
            }

            if (homeCountry != null) {
                c.getPersonalDetail().getAddress().
                                        getCountry().
                                            setPropertyValue(homeCountry);
            }


            if (otherStreet != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getStreet().
                                            setPropertyValue(otherStreet);
            }

            if (otherCity != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getCity().
                                            setPropertyValue(otherCity);
            }

            if (otherState != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getState().
                                            setPropertyValue(otherState);
            }

            if (otherPostalCode != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getPostalCode().
                                            setPropertyValue(otherPostalCode);
            }

            if (homePostOfficeBox != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getPostOfficeAddress().
                                            setPropertyValue(homePostOfficeBox);
            }


            if (otherCountry != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getCountry().
                                            setPropertyValue(otherCountry);
            }


            if (officeStreet != null) {
                c.getBusinessDetail().getAddress().
                                        getStreet().
                                            setPropertyValue(officeStreet);
            }

            if (officeCity != null) {
                c.getBusinessDetail().getAddress().
                                        getCity().
                                            setPropertyValue(officeCity);
            }

            if (officeState != null) {
                c.getBusinessDetail().getAddress().
                                        getState().
                                            setPropertyValue(officeState);
            }

            if (officePostalCode != null) {
                c.getBusinessDetail().getAddress().
                                        getPostalCode().
                                            setPropertyValue(officePostalCode);
            }

            if (officePostOfficeBox != null) {
                c.getBusinessDetail().getAddress().
                                        getPostOfficeAddress().
                                            setPropertyValue(officePostOfficeBox);
            }

            if (officeCountry != null) {
                c.getBusinessDetail().getAddress().
                                        getCountry().
                                            setPropertyValue(officeCountry);
            }

            if (notes!=null)
            {
                com.funambol.foundation.pdi.contact.Note note = new com.funambol.foundation.pdi.contact.Note();
                note.setPropertyValue(notes);
                note.setPropertyType(FIELD_TYPE_NOTES);
                c.addNote(note);
            }


        } catch (Exception e) {
            throw new DataAccessException("Error parsing contact item", e);
        }

        return c;

    }

    /**
     * get contacts from Exchange Server with the given clause
     *
     * @param username
     * @param credentials
     * @param clause
     * @param exchangeFolder
     * @return array of find contacts
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    private Contact[] getContacts (String    username    ,
                                   String    credentials ,
                                   String    clause      ,
                                   String    exchangeFolder)
            throws DataAccessException {

        Contact[] exchangeContacts    ;

        String response               ;
        String webDavContactMsg       ;
        String webDavHeaderMsg        ;

        String server                 ;
        String resource               ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        webDavContactMsg =
            MessageFormat.format(WEBDAV_MSG_SELECT_CONTACTS,
            new Object[] {server, username, resource, clause});

        webDavHeaderMsg =
            MessageFormat.format(ItemDAO.WEBDAV_HEADER_SELECT,

            new Object[] {
                        "/"           +
                            server    +
                            "/"       +
                            username  +
                            "/"       +
                            resource

            }
        );


        try {
             response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials, webDavContactMsg, FILE_ENCODING);
        } catch (Exception e) {
             throw new DataAccessException("Error getting Exchange server response",
                                           e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                       +
                                          username                      +
                                          " URI "                       +
                                          exchangeFolder                +
                                          " getting Exchange contact"   ,
                                          e)                            ;

        }

        exchangeContacts = getContactsFromWebDavMsg(response);

        return exchangeContacts;

    }

}