/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.exchange.items.contact.manager;

import com.funambol.foundation.pdi.contact.Name;
import com.funambol.framework.server.store.NotFoundException;

import com.funambol.exchange.DataAccessException;
import com.funambol.exchange.items.common.dao.ItemDAO;
import com.funambol.exchange.items.common.model.Item;
import com.funambol.exchange.items.contact.dao.ContactDAO;
import com.funambol.exchange.items.contact.model.Contact;
import com.funambol.exchange.util.StringTools;

import java.util.TimeZone;

/**
 * This class defines methods to access contacts data
 * in exchange server datastore
 *
 * @version $Id: ContactManager.java,v 1.1 2006/08/08 07:49:03 gibi Exp $
 *
 **/
public class ContactManager {

    //------------------------------------------------------------- Private Data

    private ContactDAO cd = null ;
    private ItemDAO    id = null ;

    //--------------------------------------------------------------- Costructor

    public ContactManager(String host           ,
                          int    port           ,
                          String exchangeServer,
                          TimeZone deviceTimeZone)
        throws DataAccessException {

        this.cd = new ContactDAO (host           ,
                                  port           ,
                                  exchangeServer ,
                                   deviceTimeZone);

        this.id = new ItemDAO (host,
                               port,
                               exchangeServer);

    }

    //----------------------------------------------------------- Public methods

    /**
     * get all items about principalId, sourceURI
     *
     * @param principalId
     * @param sourceURI
     *
     * @return items about principalId, sourceURI
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Item[] getAllItems(String   sourceURI ,
                              long     principalId)
    throws DataAccessException
    {

        return id.getLocalItems(sourceURI,
                                principalId);
    }

     /**
      * get contact item twins
      *
      * @param contact the contact object
      * @param principalId
      * @param sourceURI
      *
      * @return find contact item twins
      *
      * @throws com.funambol.exchange.DataAccessException
      **/
     public Item[] getTwinItems(Contact  contact     ,
                                long     principalId ,
                                String   sourceURI)
     throws DataAccessException {


        String firstName = null;
        String lastName  = null;

        Name name = contact.getName () ;

        if (name != null) {
            if (name.getFirstName() != null) {
                firstName   = name.getFirstName        ().
                                        getPropertyValueAsString()         ;
            }
            if (name.getLastName() != null) {
                lastName    = name.getLastName         ().
                                        getPropertyValueAsString()         ;
            }
        }

        long crc = StringTools.getCRCFromString(firstName + lastName);
        return id.getTwins(crc           ,
                           principalId   ,
                           sourceURI);
     }

    /**
     * get contact by key
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find contact
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Contact getContactById(String username    ,
                                  String credentials ,
                                  String id          ,
                                  String exchangeFolder   )
        throws DataAccessException, NotFoundException  {

        Contact[] contacts;

        contacts = this.cd.getContacts(username          ,
                                       credentials       ,
                                       new String[] {id} ,
                                       exchangeFolder         );

        if (contacts == null || !(contacts.length > 0)) {
            throw new NotFoundException("Contact with id " + id + " not found");
        }

        return contacts[0];

    }

    /**
     * get Contact array by username, credentials, ids, exchangeFolder
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return find contacts
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Contact[] getContacts(String username    ,
                                 String credentials ,
                                 String ids[]       ,
                                 String exchangeFolder   )
        throws DataAccessException {

        Contact[] contacts;

        contacts = this.cd.getContacts(username        ,
                                       credentials     ,
                                       ids             ,
                                       exchangeFolder);

        return contacts;

    }

    /**
     * get Contact array with setting fields only for twins
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find contacts
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Contact[] getContactsForTwin(String   username       ,
                                        String   credentials    ,
                                        String[] ids            ,
                                        String   exchangeFolder
                                        )
    throws DataAccessException {

        return cd.getContactsForTwin(username    ,
                                     credentials ,
                                     ids         ,
                                     exchangeFolder);
    }

    /**
     * add contact
     *
     * @param contact
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new contact
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Contact setContact(Contact contact     ,
                              String  username    ,
                              String  credentials ,
                              String  exchangeFolder   )
        throws DataAccessException {

        return this.cd.setContact(contact     ,
                                  username    ,
                                  credentials ,
                                  exchangeFolder   );

    }

    /**
     * remove contact
     *
     * @param contact
     * @param username
     * @param principal
     * @param exchangeFolder
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public void removeContact(Contact contact   ,
                              String  username  ,
                              String  principal ,
                              String  exchangeFolder )
        throws DataAccessException {

        this.cd.removeContact(contact   ,
                              username  ,
                              principal ,
                              exchangeFolder );

    }

}
