/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.exchange.items.note.manager;

import com.funambol.exchange.DataAccessException;
import com.funambol.exchange.items.common.dao.ItemDAO;
import com.funambol.exchange.items.common.model.Item;
import com.funambol.exchange.items.note.dao.NoteDAO;
import com.funambol.exchange.items.note.model.Note;
import com.funambol.exchange.util.StringTools;

import com.funambol.framework.server.store.NotFoundException;

/**
 * This class defines methods to access notes data
 * in exchange server datastore
 *
 * @version $Id: NoteManager.java,v 1.1 2006/08/08 07:50:04 gibi Exp $
 */
public class NoteManager {

    //------------------------------------------------------------- Private Data

    private NoteDAO nd = null;

    private ItemDAO id = null;

    //--------------------------------------------------------------- Costructor

    public NoteManager(String host,
                       int port,
                       String exchangeServer)
            throws DataAccessException {

        this.nd = new NoteDAO(host,
                port,
                exchangeServer);

        this.id = new ItemDAO(host,
                port,
                exchangeServer);

    }

    //----------------------------------------------------------- Public methods

    /**
     * get all items about principalId, sourceURI
     *
     * @param principalId
     * @param sourceURI
     * @return items about principalId, sourceURI
     * @throws com.funambol.exchange.DataAccessException
     *
     */
    public Item[] getAllItems(String sourceURI,
                              long principalId)
            throws DataAccessException {

        return id.getLocalItems(sourceURI,
                principalId);
    }

    /**
     * get note twin item array
     *
     * @param note        the note object
     * @param principalId
     * @param sourceURI
     * @return note twin item array
     * @throws com.funambol.exchange.DataAccessException
     *
     */
    public Item[] getTwinItems(Note note,
                               long principalId,
                               String sourceURI)
            throws DataAccessException {


        String noteBody = note.getTextDescription() != null ? note.getTextDescription() : "";

        long crc = StringTools.getCRCFromString(noteBody);

        return id.getTwins(crc,
                principalId,
                sourceURI);
    }

    /**
     * get note by id
     *
     * @param id
     * @param username
     * @param credentials
     * @return find note
     * @throws com.funambol.exchange.DataAccessException
     *
     */
    public Note getNoteById(String username,
                            String credentials,
                            String id,
                            String exchangeFolder)
            throws DataAccessException, NotFoundException {

        Note[] notes;

        notes = this.nd.getNotes(username,
                credentials,
                new String[]{id},
                exchangeFolder);

        if (notes == null || !(notes.length > 0)) {
            throw new NotFoundException("Note with id " + id + " not found");
        }

        return notes[0];

    }

    /**
     * get Note array by username, credentials, ids, exchangeFolder
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     * @return find calendars
     * @throws com.funambol.exchange.DataAccessException
     *
     */
    public Note[] getNotes(String username,
                           String credentials,
                           String ids[],
                           String exchangeFolder)
            throws DataAccessException {

        Note[] notes;

        notes = this.nd.getNotes(username,
                credentials,
                ids,
                exchangeFolder);

        return notes;

    }

    /**
     * get Note array with setting fields only for twins
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     * @return array of find items
     * @throws com.funambol.exchange.DataAccessException
     *
     */
    public Note[] getNotesForTwin(String username,
                                  String credentials,
                                  String[] ids,
                                  String exchangeFolder)
            throws DataAccessException {

        return nd.getNotesForTwin(username,
                credentials,
                ids,
                exchangeFolder);
    }

    /**
     * add note
     *
     * @param note
     * @param username
     * @param credentials
     * @param exchangeFolder
     * @return new note
     * @throws com.funambol.exchange.DataAccessException
     *
     */
    public Note addNote(Note note,
                        String username,
                        String credentials,
                        String exchangeFolder)
            throws DataAccessException {

        return this.nd.setNote(note,
                username,
                credentials,
                exchangeFolder,
                false);

    }

    /**
     * update note
     *
     * @param note
     * @param username
     * @param credentials
     * @param exchangeFolder
     * @return new note
     * @throws com.funambol.exchange.DataAccessException
     *
     */
    public Note updateNote(Note note,
                        String username,
                        String credentials,
                        String exchangeFolder)
            throws DataAccessException {

        return this.nd.setNote(note,
                username,
                credentials,
                exchangeFolder,
                true);

    }

    /**
     * remove note
     *
     * @param note
     * @param username
     * @param exchangeFolder
     * @throws com.funambol.exchange.DataAccessException
     *
     */
    public void removeNote(Note note,
                           String username,
                           String credentials,
                           String exchangeFolder)
            throws DataAccessException {

        this.nd.removeNote(note, username, credentials, exchangeFolder);

    }

}
