/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.exchange.items.note.model;

import java.text.MessageFormat;

import java.util.Date;

import com.funambol.exchange.items.note.NoteParseException;
import com.funambol.exchange.util.DateTools;
import com.funambol.exchange.util.StringTools;
import com.funambol.exchange.xml.XmlParser;
import com.funambol.exchange.xml.XmlParseException;

/**
 *
 * @version $Id: Note.java,v 1.1 2006/08/08 07:50:18 gibi Exp $
 **/
public class Note {

    //---------------------------------------------------------------- Constants

    private static final String TAG_PP_DATE    = "Date"   ;
    private static final String TAG_PP_SUBJECT = "Subject";
    private static final String TAG_PP_NOTE    = "Body"   ;

    private static final String MSG_PP =
          "<note>\r\n"                 +
          "<Subject>{0}</Subject>\r\n" +
          "<Body>{1}</Body>\r\n"       +
          "<Date>{2}</Date>\r\n"       +
          "</note>"                    ;

    //------------------------------------------------------------- Private data

    private String     id                  = null  ;

    private String     username            = null  ;

    private String     subject             = null  ;

    private String     textDescription     = null  ;

    private Date       date                = null  ;

    private String     href                = null  ;

    private Date       lastModified        = null  ;
    private char       state               = ' '   ;

    //------------------------------------------------------------- Constructors

    /**
     * Creates a Note
     *
     */
    public Note () {

    }

    /**
     * Creates a Note
     *
     *
     * @param id client item id
     *
     */
    public Note (String id) {

        this.id = id ;

    }

    /**
     * Creates a Note
     *
     *
     * @param id client item key
     * @param t client item timestamp
     *
     */
    public Note (String id, Date t)
    {

        this.id            = id ;
        this.lastModified  = t  ;

    }

    /**
     * Creates a Note
     *
     *
     * @param id client item id
     * @param state client item state
     *
     */
    public Note (String id, char state) {

        this.id      = id     ;
        this.state   = state  ;

    }

    /**
     * Creates a Note
     *
     *
     * @param id
     * @param href
     * @param state
     * @param lastModified
     *
     */
    public Note (String     id    ,
                 String     href  ,
                 char       state ,
                 Date       lastModified) {

        this.id            = id            ;
        this.href          = href          ;
        this.state         = state         ;
        this.lastModified  = lastModified  ;

    }

    /**
     * Creates a Note
     *
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public Note (String id, String content, Date t)
    throws NoteParseException {

        getPPNote        (id, content, t);

    }

    //----------------------------------------------------------- Public methods

    public String getId() {
        return this.id;
    }

    public String getUsername() {
            return this.username;
    }

    public String getSubject() {
        return this.subject;
    }

    public String getTextDescription() {
        return this.textDescription;
    }

    public Date getDate() {
        return this.date;
    }


    public Date getLastModified() {
        return this.lastModified;
    }

    public String getHref() {
        return this.href;
    }

    public char getState() {
        return this.state;
    }

    public void setId(String id) {
        this.id = id;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setSubject(String subject) {
        this.subject = subject;
    }

    public void setTextDescription(String textDescription) {
        this.textDescription = textDescription;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    public void setHref(String href) {
            this.href = href;
    }

    public void setState(char state) {
        this.state = state;
    }

    /**
     * Set a Note properties from Client format content
     *
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public void getPPNote (String id, String content, Date t)
    throws NoteParseException
    {

        String dt;

        this.id            = id ;
        this.lastModified  = t   ;

        try {

            dt = XmlParser.getXMLInitTagValue(content, TAG_PP_DATE );

            this.date    = DateTools.clientDateToDate(dt);

            this.subject = StringTools.unescapeXml(XmlParser.getXMLInitTagValue(content, TAG_PP_SUBJECT));

            this.textDescription = StringTools.unescapeXml(XmlParser.getXMLInitTagValue(content, TAG_PP_NOTE));

        } catch (NumberFormatException e) {
            throw new NoteParseException(e.getMessage());
        } catch (XmlParseException e) {
            throw new NoteParseException(e.getMessage());
        }

    }

    /**
     * Set a Note properties to Client format content
     *
     * @return content in client format
     *
     */
    public String toXML()
    throws NoteParseException {

         String ppMsg;
         String date;
         try {

             date = DateTools.toClientDate (this.date) ;

             ppMsg = MessageFormat.format(MSG_PP,
                new Object[] {
                                XmlParser.getString(subject         ) ,
                                XmlParser.getString(textDescription ) ,
                                XmlParser.getString(date            )

                              }
             );

         } catch (Exception e) {
             throw new NoteParseException ("Error composing calendar Device: " +
                                           e.getMessage());
         }

         return ppMsg;

     }

}
