/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.exchange.items.task.dao;

import java.text.MessageFormat;

import java.util.ArrayList;

import com.funambol.exchange.DataAccessException;
import com.funambol.exchange.items.common.dao.ItemDAO;
import com.funambol.exchange.items.task.model.Task;
import com.funambol.exchange.httptransport.WebDavHttpTransport;
import com.funambol.exchange.util.DateTools;
import com.funambol.exchange.util.StringTools;
import com.funambol.exchange.xml.XmlParser;
import com.funambol.exchange.xml.XmlParseException;

/**
 * This class implements methods to access task data in exchange
 * server datastore
 *
 * @version $Id: TaskDAO.java,v 1.1 2006/08/08 07:50:55 gibi Exp $
 *
 **/
public class TaskDAO extends ItemDAO {

    //---------------------------------------------------------------- Constants

    private static final String WEBDAV_MSG_SELECT_TASKS                       =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                           +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                   +
    "<D:sql>\n"                                                               +
    "Select "                                                                 +
    "\"urn:schemas:httpmail:textdescription\" AS textdescription, "           +
    "\"urn:schemas:httpmail:subject\" AS subject, "                           +
    "\"urn:schemas:calendar:dtstart\" AS dtstart, "                           +
    "\"urn:schemas:calendar:dtend\" AS dtend, "                               +
    "\"http://schemas.microsoft.com/exchange/keywords-utf8\" AS categories, " +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8110\" as actualwork, "         +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062008-0000-0000-C000-000000000046'}'/0x8535\" as billinginformation, " +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062008-0000-0000-C000-000000000046'}'/0x8539\" as companies, "          +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x811c\" as complete, "           +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8104\" as date, "               +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x810F\" as datecompleted, "      +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8105\" as duedate, "            +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x811F\" as owner, "              +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8102\" as perc, "               +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8101\" as status, "             +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8103\" as TeamTask, "           +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8111\" as TotalWork, "          +
    "\"http://schemas.microsoft.com/mapi/reminderset\" as reminderset, "      +
    "\"http://schemas.microsoft.com/mapi/remindertime\" as remindertime, "    +
    "\"urn:schemas:httpmail:importance\" as importance, "                     +
    "\"http://schemas.microsoft.com/exchange/mileage\" as mileage, "          +
    "\"http://schemas.microsoft.com/mapi/sensitivity\" as sensitivity, "      +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "             +
    "\"DAV:getlastmodified\" AS getlastmodified, "                            +
    "\"DAV:href\" AS href, "                                                  +
    "\"DAV:isfolder\" AS isfolder "                                           +
    "FROM \"/{0}/{1}/{2}\"  "                                                 +
    "{3} "                                                                    +
    "</D:sql>\n"                                                              +
    "</D:searchrequest>"                                                      ;

    private static final String WEBDAV_MSG_SELECT_TASKS_FOR_TWIN              =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                           +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                   +
    "<D:sql>\n"                                                               +
    "Select "                                                                 +
    "\"urn:schemas:httpmail:subject\" AS subject, "                           +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8104\" as date, "               +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8105\" as duedate, "            +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "             +
    "\"DAV:getlastmodified\" AS getlastmodified, "                            +
    "\"DAV:creationdate\" AS creationdate, "                                  +
    "\"DAV:href\" AS href, "                                                  +
    "\"DAV:isfolder\" AS isfolder "                                           +
    "FROM \"/{0}/{1}/{2}\"  "                                                 +
    "{3}"                                                                     +
    "</D:sql>\n"                                                              +
    "</D:searchrequest>"                                                      ;

    public static final String TAG_DATE               = "date"             ;
    public static final String TAG_DESCRIPTION        =
        "textdescription"                                                  ;

    public static final String TAG_RESPONSE           = "a:response"       ;
    public static final String TAG_SUBJECT            = "subject"          ;

    public static final String TAG_REPLUID            = "repluid"          ;

    public static final String TAG_LAST_MODIFIED      = "getlastmodified"  ;

    public static final String TAG_DUE_DATE           = "duedate"          ;


    //------------------------------------------------------------- Private data

    private WebDavHttpTransport  webDavHttp          = null ;

    //-------------------------------------------------------------- Costructors

    public TaskDAO(String host           ,
                   int    port           ,
                   String exchangeServer)
        throws DataAccessException {

        this.webDavHttp = new WebDavHttpTransport(host           ,
                                                  port           ,
                                                  exchangeServer);
    }
    //----------------------------------------------------------- Public methods

    /**
     * insert / update task
     *
     * @param task
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new / updated task
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Task setTask(Task    task           ,
                        String  username       ,
                        String  credentials    ,
                        String  exchangeFolder )
        throws DataAccessException {

        String response                ;

        String id                      ;

        StringBuffer webDavTaskMsg     = new StringBuffer();

        String webDavHeaderMsg         ;

        String server                  ;
        String resource                ;

        String tmp                     ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        webDavTaskMsg.append("<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n");
        webDavTaskMsg.append("<D:propertyupdate xmlns:D=\"DAV:\" ");
        webDavTaskMsg.append("xmlns:EX=\"http://schemas.microsoft.com/exchange/\" ");
        webDavTaskMsg.append("xmlns:HN=\"urn:schemas:httpmail:\" ");
        webDavTaskMsg.append("xmlns:MAPI_ID_1=\"http://schemas.microsoft.com/mapi/id/{00062003-0000-0000-C000-000000000046}/\" ");
        webDavTaskMsg.append("xmlns:MAPI_ID_2=\"http://schemas.microsoft.com/mapi/id/{00062008-0000-0000-C000-000000000046}/\" ");
        webDavTaskMsg.append("xmlns:MAPI=\"http://schemas.microsoft.com/mapi/\" ");
        webDavTaskMsg.append("xmlns:x=\"xml:\" ");
        webDavTaskMsg.append("xmlns:b=\"urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/\">\n");
        webDavTaskMsg.append("<D:set>\n");
        webDavTaskMsg.append(  "<D:prop>\n");
        webDavTaskMsg.append("<D:contentclass>urn:content-classes:person</D:contentclass>\n");
        webDavTaskMsg.append("<EX:outlookmessageclass>IPM.Task</EX:outlookmessageclass>\n");

        if (task.getActualWork() != null) {
            webDavTaskMsg.append("<MAPI_ID_1:0x8110 b:dt=\"int\">");
            webDavTaskMsg.append(task.getActualWork());
            webDavTaskMsg.append("</MAPI_ID_1:0x8110>");
        }

        tmp = DateTools.dateToWebDavTag(task.getDate());
        if (tmp != null && !tmp.equals("")) {
            webDavTaskMsg.append("<MAPI_ID_1:0x8104 b:dt=\"dateTime.tz\">");
            webDavTaskMsg.append(tmp);
            webDavTaskMsg.append("</MAPI_ID_1:0x8104>");
        }

        tmp = DateTools.dateToWebDavTag(task.getDueDate());
        if (tmp != null && !tmp.equals("")) {
            webDavTaskMsg.append("<MAPI_ID_1:0x8105 b:dt=\"dateTime.tz\">");
            webDavTaskMsg.append(tmp);
            webDavTaskMsg.append("</MAPI_ID_1:0x8105>");
        }

        String billingInformation = task.getBillingInformation();
        if (billingInformation != null) {
            billingInformation  = StringTools.escapeXml(billingInformation);
            webDavTaskMsg.append("<MAPI_ID_2:0x8535>");
            webDavTaskMsg.append(billingInformation);
            webDavTaskMsg.append("</MAPI_ID_2:0x8535>");
        }

        String companies = task.getCompanies();
        if (companies != null && companies.length() > 0) {

            companies  = StringTools.escapeXml(companies);

            companies = XmlParser.
                            fieldToMultipleTag
                                (companies);

            webDavTaskMsg.append("<MAPI_ID_2:0x8539 b:dt=\"mv.string\">");
            webDavTaskMsg.append(companies);
            webDavTaskMsg.append("</MAPI_ID_2:0x8539>");
        }

        if (task.getImportance() != null) {
            webDavTaskMsg.append("<HN:importance>");
            webDavTaskMsg.append(task.getImportance());
            webDavTaskMsg.append("</HN:importance>");
        }

        if (task.getMileage() != null) {
            webDavTaskMsg.append("<EX:mileage>");
            webDavTaskMsg.append(task.getMileage());
            webDavTaskMsg.append("</EX:mileage>");
        }

        String owner = task.getOwner();
        if (owner != null) {
            owner  = StringTools.escapeXml(owner);
            webDavTaskMsg.append("<MAPI_ID_1:0x811F>");
            webDavTaskMsg.append(owner);
            webDavTaskMsg.append("</MAPI_ID_1:0x811F>");
        }

        String categories = task.getCategories();
        if (categories != null && categories.length() > 0) {

            categories  = StringTools.escapeXml(categories);

            categories = XmlParser.
                            fieldToMultipleTag
                                (categories);

            webDavTaskMsg.append("<EX:keywords-utf8>");
            webDavTaskMsg.append(categories);
            webDavTaskMsg.append("</EX:keywords-utf8>");
        }

        webDavTaskMsg.append("<MAPI:reminderset>");
        webDavTaskMsg.append((task.getReminderSet() ? "1" : "0"));
        webDavTaskMsg.append("</MAPI:reminderset>");

        tmp = DateTools.dateToWebDavTag(task.getReminderTime());
        if (tmp != null && tmp.length()>0) {
            webDavTaskMsg.append("<MAPI:remindertime b:dt=\"dateTime.tz\">");
            webDavTaskMsg.append(tmp);
            webDavTaskMsg.append("</MAPI:remindertime>");
        }

        if (task.getSensitivity() != null) {
            webDavTaskMsg.append("<MAPI:sensitivity>");
            webDavTaskMsg.append(task.getSensitivity());
            webDavTaskMsg.append("</MAPI:sensitivity>");
        }

        webDavTaskMsg.append("<MAPI_ID_1:0x811c b:dt=\"boolean\">");
        webDavTaskMsg.append( (task.isComplete() ? "1" : "0") );
        webDavTaskMsg.append("</MAPI_ID_1:0x811c>");


        String sPerc = task.getPercentComplete();
        String status = task.getStatus();
        String dateCompleted = null;
        if (task.getDateCompleted() == null) {
            DateTools.dateToWebDavTag(task.getDateCompleted());
        }

        if (task.isComplete()) {
            sPerc  = "100";
            status = "2";   // Completed
        }

        if (sPerc != null) {
            float perc = Float.parseFloat(sPerc) / 100;
            webDavTaskMsg.append("<MAPI_ID_1:0x8102 b:dt=\"float\">");
            webDavTaskMsg.append(perc);
            webDavTaskMsg.append("</MAPI_ID_1:0x8102>");
        }

        if (status != null) {
            webDavTaskMsg.append("<MAPI_ID_1:0x8101  b:dt=\"int\">");
            webDavTaskMsg.append(status);
            webDavTaskMsg.append("</MAPI_ID_1:0x8101>");
        }

        if (dateCompleted != null) {
            if (dateCompleted.length() > 0) {
                webDavTaskMsg.append("<MAPI_ID_1:0x810F b:dt=\"dateTime.tz\">");
                webDavTaskMsg.append(dateCompleted);
                webDavTaskMsg.append("</MAPI_ID_1:0x810F>");
            }
            else {
                webDavTaskMsg.append("<MAPI_ID_1:0x810F/>");
            }
        }

        String subject = task.getSubject();

        if (subject != null) {
            subject  = StringTools.escapeXml(subject);
            webDavTaskMsg.append("<HN:subject>");
            webDavTaskMsg.append(subject);
            webDavTaskMsg.append("</HN:subject>");
        }

        webDavTaskMsg.append("<MAPI_ID_1:0x8103 b:dt=\"boolean\">");
        webDavTaskMsg.append( (task.isTeamTask() ? "1" : "0"));
        webDavTaskMsg.append("</MAPI_ID_1:0x8103>");

        String textDescription  = task.getTextDescription();
        if (textDescription != null) {
            textDescription  = StringTools.escapeXml(textDescription);
            webDavTaskMsg.append("<HN:textdescription>");
            webDavTaskMsg.append(textDescription);
            webDavTaskMsg.append("</HN:textdescription>");
        }

        if (task.getTotalWork() != null) {
            webDavTaskMsg.append("<MAPI_ID_1:0x8111 b:dt=\"int\">");
            webDavTaskMsg.append(task.getTotalWork());
            webDavTaskMsg.append("</MAPI_ID_1:0x8111>");
        }

        webDavTaskMsg.append(" </D:prop>\n");
        webDavTaskMsg.append("</D:set>\n");
        webDavTaskMsg.append("</D:propertyupdate>");

        webDavHeaderMsg
            = MessageFormat.format(ItemDAO.WEBDAV_HEADER_PROPPATCH,
                new Object[] {
                    "/"                   +
                        server            +
                        "/"               +
                        username          +
                        "/"               +
                        resource             ,
                    task.getHref()
              }
        );


        try {
             response = this.webDavHttp.sendRequest(webDavHeaderMsg,
                                                    credentials,
                                                    webDavTaskMsg.toString(),
                                                    FILE_ENCODING);
        } catch (Exception e) {
            throw new DataAccessException("Error getting Exchange server response",
                                          e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                   +
                                          username                  +
                                          " URI "                   +
                                          exchangeFolder            +
                                          " setting exchange task"  ,
                                          e)                        ;

        }

        try {
            id = XmlParser.getRuidFromResponse(response);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error getting task id", e);
        }

        task.setId(id);

        return task;

    }

    /**
     * Delete a <i>Task</i> from Exchange server.
     *
     * @param task
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @throws com.funambol.exchange.DataAccessException
     *
     */
    public void removeTask(Task    task           ,
                           String  username       ,
                           String  credentials    ,
                           String  exchangeFolder )
        throws DataAccessException {

        String webDavTaskMsg           ;

        String webDavHeaderMsg         ;

        String response                ;

        String server                  ;
        String resource                ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        webDavTaskMsg          = ""                ;

        webDavHeaderMsg =
            MessageFormat.format(ItemDAO.WEBDAV_HEADER_REMOVE,
            new Object[] {
                        "/"                     +
                            server              +
                            "/"                 +
                            username            +
                            "/"                 +
                            resource              ,
                        task.getHref()
            }
        );

        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials, webDavTaskMsg, FILE_ENCODING);
        } catch (Exception e) {
            throw new DataAccessException("Error getting Exchange server response",
                                          e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                     +
                                          username                    +
                                          " URI "                     +
                                          exchangeFolder              +
                                          " removing exchange task"   ,
                                          e)                          ;

        }

    }

    /**
     * get tasks from Exchange Server
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find tasks
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Task[] getTasks (String    username       ,
                            String    credentials    ,
                            String[]  ids            ,
                            String    exchangeFolder )
        throws DataAccessException {

        String clause        = getClause(ids);
        return getTasks(username, credentials, clause, exchangeFolder);
    }

    /**
     * get Task array with setting fields only for twins
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     * @param ids
     *
     * @return Task array with setting fields only for twins
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Task[] getTasksForTwin(String   username       ,
                                  String   credentials    ,
                                  String[] ids            ,
                                  String   exchangeFolder
                                  )
    throws DataAccessException {

        ArrayList tasks                ;
        Task      t                    ;

        String[]  props                ;
        String    prop                 ;
        String[]  resp                 ;

        String    replUid              ;
        String    uri                  ;
        String    lastUpdate           ;

        String    subject              ;
        String    date                 ;
        String    dueDate              ;

        String    isFolder             ;

        String    response             ;

        String    webDavBodyMsg        ;

        String    webDavHeaderMsg      ;

        String    server               ;
        String    resource             ;

        int       count                = 0    ;

        String    clause               ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        clause = getClause(ids);

        webDavBodyMsg =
                    MessageFormat.format(WEBDAV_MSG_SELECT_TASKS_FOR_TWIN,
            new Object[] {server, username, resource, clause});

        webDavHeaderMsg =
            MessageFormat.format(WEBDAV_HEADER_SELECT,

            new Object[] {
                    "/"                  +
                        server           +
                        "/"              +
                        username         +
                        "/"              +
                        resource
            }
        );


        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg ,
                                                   credentials     ,
                                                   webDavBodyMsg   ,
                                                   FILE_ENCODING   );
        } catch (Exception e) {
            throw new DataAccessException("Error getting Exchange server response",
                                           e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                    +
                                          username                   +
                                          " URI "                    +
                                          exchangeFolder             +
                                          " reading exchange items"  ,
                                          e)                         ;
        }

        resp = new String[] {response};

        try {
            props = XmlParser.getXMLTag(resp, TAG_RESPONSE);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing item", e);
        }

        tasks = new ArrayList();

        for (int i=0, l = props.length; i < l; i++) {

            try {

                prop = props[i];

                isFolder      = XmlParser.getXMLInitTagValue
                                 (prop, TAG_IS_FOLDER)        ;

                replUid       = XmlParser.getXMLTagValue
                                 (prop, TAG_REPLUID)          ;

                uri           = XmlParser.getXMLTagValue
                                 (prop, TAG_HREF)             ;

                lastUpdate    = XmlParser.getXMLInitTagValue
                                 (prop, TAG_LAST_MODIFIED)    ;

                subject       = XmlParser.getXMLInitTagValue
                                 (prop, "subject")            ;

                date          = XmlParser.getXMLInitTagValue
                                 (prop, TAG_DATE);

                dueDate       = XmlParser.getXMLInitTagValue
                                 (prop, "duedate");

                t = new Task(getIdFromReplUid(replUid), DateTools.webDavTagToDate(lastUpdate));

                t.setHref( XmlParser.getHrefFromUri(uri));

                t.setDate(DateTools.webDavTagToDate(date));

                t.setDueDate(DateTools.webDavTagToDate(dueDate));

                t.setSubject(subject);

                if(PROP_NO_FOLDER.equals(isFolder)) {
                    tasks.add(t);
                    count ++;
                }

            } catch (Exception e) {
                throw new DataAccessException("Error parsing item", e);
            }

        }

        return (Task[]) tasks.toArray(new Task[count]);
    }

    /**
     * get task array from Exchange Server
     *
     * @param username
     * @param credentials
     * @param fields
     * @param values
     * @param exchangeFolder
     * @return array of find tasks
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    public Task[] getTasks(String username,
                           String credentials,
                           String fields[],
                           Object values[],
                           String exchangeFolder) throws DataAccessException {

        String clause;

        clause = getClause(fields, values);

        return getTasks(username, credentials, clause, exchangeFolder);
    }

    //---------------------------------------------------------- Private methods

    /**
     * Create a <i>Task</i> array object from WebDav msg.
     *
     * @param webDavMsg
     * @param webDavMsg
     *
     * @return a newly create <i>Task</i> object initialized with the
     *         fields in the result set
     *
     */
    private Task[] getTasksFromWebDavMsg(String webDavMsg)
    throws DataAccessException {

        ArrayList  tasks      ;
        String []  resps      ;
        String []  msg        ;

        String     response   ;
        String     id         ;
        String     replUid    ;

        String     isFolder   ;
        int        count      = 0    ;

        msg = new String[] {webDavMsg};

        tasks = new ArrayList();

        try {

            resps = XmlParser.getXMLTag(msg, TAG_RESPONSE);

            for (int i = 0, l = resps.length; i < l; i++) {

                response =  resps[i]                                           ;
                replUid  =  XmlParser.getXMLInitTagValue(response, TAG_REPLUID);
                id       =  getIdFromReplUid(replUid)                          ;

                isFolder    =  XmlParser.getXMLInitTagValue(response,
                                                            TAG_IS_FOLDER)  ;
                if(PROP_NO_FOLDER.equals(isFolder)) {
                    tasks.add(getTaskFromResponseTag(id, response));
                    count ++;
                }

            }

        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing task", e);
        }

        return (Task[])tasks.toArray(new Task[count]);

    }

    /**
     * Create a <i>Task</i> object from response tag.
     *
     * @param id
     * @param msgResponse
     *
     * @return a newly create <i>Task</i> object initialized with the
     *         fields in the result set
     *
     */
    private Task getTaskFromResponseTag(String id, String msgResponse)
    throws DataAccessException {

        Task   t            ;
        String lastUpdate   ;
        String date         ;

        String categories   ;

        String companies    ;

        String tmp          ;

        String uri          ;

        t = new Task(id);

        try {

            uri         = XmlParser.getXMLTagValue
                             (msgResponse, TAG_HREF);

            t.setHref( XmlParser.getHrefFromUri(uri));

            tmp = XmlParser.getXMLInitTagValue
                        (msgResponse, "actualwork");

            if (tmp != null && tmp.length() > 0) {
                 t.setActualWork(Integer.valueOf(tmp));
            }

            t.setBillingInformation(XmlParser.getXMLInitTagValue
                         (msgResponse, "billinginformation"));

            t.setTextDescription(XmlParser.getXMLInitTagValue
                                 (msgResponse, TAG_DESCRIPTION));

            t.setSubject(XmlParser.getXMLInitTagValue
                         (msgResponse, TAG_SUBJECT));

            date = XmlParser.getXMLInitTagValue
                   (msgResponse, TAG_DATE);

            t.setDate(DateTools.webDavTagToDate(date));

            lastUpdate = XmlParser.getXMLInitTagValue
                         (msgResponse, TAG_LAST_MODIFIED);

            t.setLastModified(DateTools.webDavTagToDate
                              (lastUpdate));

            t.setTextDescription(XmlParser.getXMLInitTagValue
                                 (msgResponse, TAG_DESCRIPTION));

            companies = XmlParser.getXMLInitTagValue
                            (msgResponse, "companies");

            if (companies != null) {

                try {
                    companies = XmlParser.getFieldFromMultipleTag(companies);
                } catch (Exception e) {
                    throw new DataAccessException(e.getMessage());
                }

                t.setCompanies(companies);

            }

            categories = XmlParser.getXMLInitTagValue
                            (msgResponse, "categories");

            if (categories != null) {

                try {
                    categories = XmlParser.getFieldFromMultipleTag(categories);
                } catch (Exception e) {
                    throw new DataAccessException(e.getMessage());
                }

                t.setCategories(categories);

            }

            t.setDueDate(DateTools.webDavTagToDate(XmlParser.getXMLInitTagValue
                                                   (msgResponse, "duedate")));

            t.setDateCompleted(DateTools.webDavTagToDate(XmlParser.getXMLInitTagValue
                                                   (msgResponse, "datecompleted")));

            t.setImportance(XmlParser.getXMLInitTagValue
                            (msgResponse, "importance"));

            t.setMileage(XmlParser.getXMLInitTagValue
                         (msgResponse, "mileage"));

            t.setOwner(XmlParser.getXMLInitTagValue
                         (msgResponse, "owner"));

            tmp = XmlParser.getXMLInitTagValue
                                 (msgResponse, "perc");


            int perc = 0;

            if (tmp != null && tmp.length() > 0) {

                Float f = new Float(tmp);

                perc = new Double(f.doubleValue() * 100.0).intValue();

            }

            t.setPercentComplete(String.valueOf(perc));

            tmp = XmlParser.getXMLInitTagValue
                             (msgResponse, "reminderset");

            if ("1".equals(tmp)) {
                t.setReminderSet(true);
            } else {
                t.setReminderSet(false);
            }

            t.setReminderTime(DateTools.webDavTagToDate(XmlParser.getXMLInitTagValue
                                                        (msgResponse, "remindertime")));

            t.setSensitivity(XmlParser.getXMLInitTagValue
                             (msgResponse, "sensitivity"));

            t.setStatus(XmlParser.getXMLInitTagValue
                        (msgResponse, "status"));

            tmp = XmlParser.getXMLInitTagValue
                             (msgResponse, "TeamTask");

            if ("1".equals(tmp)) {
                t.setTeamTask(true);
            } else {
                t.setTeamTask(false);
            }

            tmp = XmlParser.getXMLInitTagValue
                        (msgResponse, "TotalWork");

            if (tmp != null && tmp.length() > 0) {
                 t.setTotalWork(Integer.valueOf(tmp));
            }

            tmp = XmlParser.getXMLInitTagValue(msgResponse, "complete");

            if (tmp != null && tmp.equals("1")) {
                t.setComplete(true);
            } else {
                t.setComplete(false);
            }

        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing task item: ", e);
        }

        return t;

    }

    /**
     * Build a clause
     * to webdav select request
     * between ids item array
     *
     * @param fields
     * @param values
     * @return webdav select clause
     **/
    private static String getClause (String[] fields, Object[] values) {

        StringBuffer clause = new StringBuffer();

        int l = fields.length;

        if (l == 0) {
            return "";
        }

        clause.append("where ");

        StringBuffer tmp ;
        String valueTmp  ;

        for (int i = 0; i < l; i++) {
            tmp = new StringBuffer();

            if (TAG_DATE.equals(fields[i])) {
                tmp.append("\"http://schemas.microsoft.com/mapi/id/{00062003-0000-0000-C000-000000000046}/0x8104\"");

                if (values[i] != null) {
                    tmp.append("=CAST(\"");
                    valueTmp = DateTools.dateToWebDavTag((java.util.Date)values[i]);
                    tmp.append(valueTmp).append("\" as 'dateTime')");
                } else {
                    tmp.append(" is null ");
                }
            }

            if (TAG_DUE_DATE.equals(fields[i])) {
                tmp.append("\"http://schemas.microsoft.com/mapi/id/{00062003-0000-0000-C000-000000000046}/0x8105\"");

                if (values[i] != null) {
                    tmp.append("=CAST(\"");
                    valueTmp = DateTools.dateToWebDavTag((java.util.Date)values[i]);
                    tmp.append(valueTmp).append("\" as 'dateTime')");
                } else {
                    tmp.append(" is null ");
                }
            }


            if (TAG_SUBJECT.equals(fields[i])) {
                tmp.append("\"urn:schemas:httpmail:subject\"");

                if (values[i] != null) {
                    tmp.append("='");
                    valueTmp = String.valueOf(values[i]);
                    valueTmp = valueTmp.replaceAll("'","''");
                    valueTmp = StringTools.escapeXml(valueTmp);
                    tmp.append(valueTmp).append("'");
                } else {
                    tmp.append(" is null ");
                }

            }

            if (i != 0 && tmp.length() != 0) {
                clause.append(" and ").append(tmp);
            } else if (tmp.length() != 0) {
                clause.append(tmp);
            }

        }

        clause.append("\r\n");

        return clause.toString();

    }

    /**
     * get tasks from Exchange Server with the given clause
     *
     * @param username
     * @param credentials
     * @param clause
     * @param exchangeFolder
     * @return array of find task
     *
     * @throws com.funambol.exchange.DataAccessException
     **/
    private Task[] getTasks(String username,
                            String credentials,
                            String clause,
                            String exchangeFolder) throws DataAccessException {

        Task[] exchangeTasks   ;

        String response        ;
        String webDavTaskMsg   ;

        String webDavHeaderMsg ;

        String server          ;
        String resource        ;

        server   = getServerFromExchangeFolder(exchangeFolder);
        resource = getResourceFromExchangeFolder(exchangeFolder);
        webDavTaskMsg =
                MessageFormat.format(WEBDAV_MSG_SELECT_TASKS,
                                     new Object[] {server, username, resource,
                                     clause});

        webDavHeaderMsg =
                MessageFormat.format(ItemDAO.WEBDAV_HEADER_SELECT,
                                         new Object[] {
                                             "/" +
                                             server +
                                             "/" +
                                             username +
                                             "/" +
                                             resource
                                         }
                                     );

        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials,
                    webDavTaskMsg, FILE_ENCODING);
        } catch (Exception e) {
            throw new DataAccessException(
                    "Error getting Exchange server response",
                    e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER " +
                                          username +
                                          " URI " +
                                          exchangeFolder +
                                          " getting exchange task",
                                          e);

        }

        exchangeTasks = getTasksFromWebDavMsg(response);

        return exchangeTasks;
    }

}
