/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.exchange.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;

import javax.naming.InitialContext;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;

import com.funambol.exchange.DataAccessException;

/**
 * This class provide methods to access database source
 *
 * @version $Id: DataAccess.java,v 1.1 2006/08/08 07:46:13 gibi Exp $
 **/
public class DataAccess {

    // --------------------------------------------------------------- Constants

    private static final String JNDI_DATASOURCE_NAME_BUNDLE = 
    "java:comp/env/jdbc/fnblds";
    private static final String JNDI_DATASOURCE_NAME_JBOSS  = 
    "java:/jdbc/fnblds"        ;

    // ------------------------------------------------------------ Private data

    private String jndiDataSourceName = null;

    // ---------------------------------------------------------- Protected data

    /**
     * Data store to use for database connections
     */
    protected transient DataSource dataSource = null;

    // -------------------------------------------------------------- Costructor

    /** Creates a new instance of DataAccess */
    public DataAccess()
    throws DataAccessException {
        setJndiDataSourceName();
    }

    // ---------------------------------------------------------- Public methods

    public void setJndiDataSourceName()
    throws DataAccessException {

        try {

            InitialContext ctx = new InitialContext();

            try {
                jndiDataSourceName = JNDI_DATASOURCE_NAME_JBOSS  ;
                dataSource = (DataSource) ctx.lookup(jndiDataSourceName ) ;
            } catch (NameNotFoundException e) {
                jndiDataSourceName = JNDI_DATASOURCE_NAME_BUNDLE ;
                dataSource = (DataSource) ctx.lookup(jndiDataSourceName ) ;
            }

        } catch (NamingException e) {
            throw new DataAccessException ("Data source "
            + jndiDataSourceName
            + " not found "
            + e.toString()
            );
        }
    }

    /**
     * @return database connection
     *
     * @throws DataAccessException
     **/
    public Connection getConnection ()
    throws DataAccessException {

        Connection con = null;

        try {

            con = this.dataSource.getConnection();

        } catch (SQLException e) {
            throw new DataAccessException
                ("Error creating connection: " + e.getMessage());
        }

        return con;

    }

    /**
     * cleanUp database connection
     *
     * @param con coonection
     * @param ps  preparement statement
     * @param rs  resultset
     *
     * @throws DataAccessException
     **/
    public void cleanUp (Connection        con ,
                         PreparedStatement ps  ,
                         ResultSet         rs  )
    throws DataAccessException {

        try {

            if (con != null) {
                con.close();
                con = null;
            }

            if (ps != null) {
                ps.close();
                ps = null;
            }

            if (rs != null) {
                rs.close();
                rs = null;
            }

        } catch (SQLException e) {
            throw new DataAccessException
                ("Error cleaning up connection: " + e.getMessage());
        }

    }

}